@@ -150,6 +150,15 @@ AVInputFormat *av_find_input_format(const char *short_name)
 
 /* memory handling */
 
+/**
+ * Default packet destructor 
+ */
+static void av_destruct_packet(AVPacket *pkt)
+{
+    av_free(pkt->data);
+    pkt->data = NULL; pkt->size = 0;
+}
+
 /**
  * Allocate the payload of a packet and intialized its fields to default values.
  *
@@ -159,34 +168,18 @@ AVInputFormat *av_find_input_format(const char *short_name)
  */
 int av_new_packet(AVPacket *pkt, int size)
 {
-    int i;
-    pkt->data = av_malloc(size + FF_INPUT_BUFFER_PADDING_SIZE);
-    if (!pkt->data)
+    void *data = av_malloc(size + FF_INPUT_BUFFER_PADDING_SIZE);
+    if (!data)
         return AVERROR_NOMEM;
+    memset(data + size, 0, FF_INPUT_BUFFER_PADDING_SIZE);
+
+    av_init_packet(pkt);
+    pkt->data = data; 
     pkt->size = size;
-    /* sane state */
-    pkt->pts = AV_NOPTS_VALUE;
-    pkt->stream_index = 0;
-    pkt->flags = 0;
-    
-    for(i=0; i<FF_INPUT_BUFFER_PADDING_SIZE; i++)
-        pkt->data[size+i]= 0;
-
+    pkt->destruct = av_destruct_packet;
     return 0;
 }
 
-/**
- * Free a packet
- *
- * @param pkt packet to free
- */
-void av_free_packet(AVPacket *pkt)
-{
-    av_freep(&pkt->data);
-    /* fail safe */
-    pkt->size = 0;
-}
-
 /* fifo handling */
 
 int fifo_init(FifoBuffer *f, int size)
@@ -204,7 +197,7 @@ void fifo_free(FifoBuffer *f)
     av_free(f->buffer);
 }
 
-int fifo_size(FifoBuffer *f, UINT8 *rptr)
+int fifo_size(FifoBuffer *f, uint8_t *rptr)
 {
     int size;
 
@@ -217,9 +210,9 @@ int fifo_size(FifoBuffer *f, UINT8 *rptr)
 }
 
 /* get data from the fifo (return -1 if not enough data) */
-int fifo_read(FifoBuffer *f, UINT8 *buf, int buf_size, UINT8 **rptr_ptr)
+int fifo_read(FifoBuffer *f, uint8_t *buf, int buf_size, uint8_t **rptr_ptr)
 {
-    UINT8 *rptr = *rptr_ptr;
+    uint8_t *rptr = *rptr_ptr;
     int size, len;
 
     if (f->wptr >= rptr) {
@@ -245,10 +238,10 @@ int fifo_read(FifoBuffer *f, UINT8 *buf, int buf_size, UINT8 **rptr_ptr)
     return 0;
 }
 
-void fifo_write(FifoBuffer *f, UINT8 *buf, int size, UINT8 **wptr_ptr)
+void fifo_write(FifoBuffer *f, uint8_t *buf, int size, uint8_t **wptr_ptr)
 {
     int len;
-    UINT8 *wptr;
+    uint8_t *wptr;
     wptr = *wptr_ptr;
     while (size > 0) {
         len = f->end - wptr;
@@ -478,7 +471,7 @@ int av_find_stream_info(AVFormatContext *ic)
     AVFrame picture;
     AVPacketList *pktl=NULL, **ppktl;
     short samples[AVCODEC_MAX_AUDIO_FRAME_SIZE / 2];
-    UINT8 *ptr;
+    uint8_t *ptr;
     int min_read_size, max_read_size;
 
     /* typical mpeg ts rate is 40 Mbits. DVD rate is about 10
@@ -612,7 +605,7 @@ int av_find_stream_info(AVFormatContext *ic)
                            TS case we do not do it because it would be
                            too long */
                         st->codec_info_nb_real_frames++;
-                        st->codec_info_nb_repeat_frames += st->codec.repeat_pict;
+                        st->codec_info_nb_repeat_frames += st->codec.coded_frame->repeat_pict;
 #if 0
                         /* XXX: testing */
                         if ((st->codec_info_nb_real_frames % 24) == 23) {
@@ -766,11 +759,11 @@ int av_write_header(AVFormatContext *s)
         switch (st->codec.codec_type) {
         case CODEC_TYPE_AUDIO:
             av_frac_init(&st->pts, 0, 0, 
-                         (INT64)s->pts_num * st->codec.sample_rate);
+                         (int64_t)s->pts_num * st->codec.sample_rate);
             break;
         case CODEC_TYPE_VIDEO:
             av_frac_init(&st->pts, 0, 0, 
-                         (INT64)s->pts_num * st->codec.frame_rate);
+                         (int64_t)s->pts_num * st->codec.frame_rate);
             break;
         default:
             break;
@@ -793,7 +786,7 @@ int av_write_frame(AVFormatContext *s, int stream_index, const uint8_t *buf,
                    int size)
 {
     AVStream *st;
-    INT64 pts_mask;
+    int64_t pts_mask;
     int ret, frame_size;
 
     st = s->streams[stream_index];
@@ -823,11 +816,11 @@ int av_write_frame(AVFormatContext *s, int stream_index, const uint8_t *buf,
             frame_size = st->codec.frame_size;
         }
         av_frac_add(&st->pts, 
-                    (INT64)s->pts_den * frame_size);
+                    (int64_t)s->pts_den * frame_size);
         break;
     case CODEC_TYPE_VIDEO:
         av_frac_add(&st->pts, 
-                    (INT64)s->pts_den * FRAME_RATE_BASE);
+                    (int64_t)s->pts_den * FRAME_RATE_BASE);
         break;
     default:
         break;
@@ -921,16 +914,16 @@ int parse_image_size(int *width_ptr, int *height_ptr, const char *str)
     return 0;
 }
 
-INT64 av_gettime(void)
+int64_t av_gettime(void)
 {
 #ifdef CONFIG_WIN32
     struct _timeb tb;
     _ftime(&tb);
-    return ((INT64)tb.time * INT64_C(1000) + (INT64)tb.millitm) * INT64_C(1000);
+    return ((int64_t)tb.time * int64_t_C(1000) + (int64_t)tb.millitm) * int64_t_C(1000);
 #else
     struct timeval tv;
     gettimeofday(&tv,NULL);
-    return (INT64)tv.tv_sec * 1000000 + tv.tv_usec;
+    return (int64_t)tv.tv_sec * 1000000 + tv.tv_usec;
 #endif
 }
 
@@ -962,10 +955,10 @@ static time_t mktimegm(struct tm *tm)
  *  HH[:MM[:SS[.m...]]]
  *  S+[.m...]
  */
-INT64 parse_date(const char *datestr, int duration)
+int64_t parse_date(const char *datestr, int duration)
 {
     const char *p;
-    INT64 t;
+    int64_t t;
     struct tm dt;
     int i;
     static const char *date_fmt[] = {
@@ -1034,7 +1027,7 @@ INT64 parse_date(const char *datestr, int duration)
         if (duration)
             return 0;
         else
-            return now * INT64_C(1000000);
+            return now * int64_t_C(1000000);
     }
 
     if (duration) {
@@ -1174,7 +1167,7 @@ int get_frame_filename(char *buf, int buf_size,
  * @param buf buffer
  * @param size buffer size
  */
-void av_hex_dump(UINT8 *buf, int size)
+void av_hex_dump(uint8_t *buf, int size)
 {
     int len, i, j, c;
 
@@ -1278,7 +1271,7 @@ void av_set_pts_info(AVFormatContext *s, int pts_wrap_bits,
  * @param num must be >= 0
  * @param den must be >= 1 
  */
-void av_frac_init(AVFrac *f, INT64 val, INT64 num, INT64 den)
+void av_frac_init(AVFrac *f, int64_t val, int64_t num, int64_t den)
 {
     num += (den >> 1);
     if (num >= den) {
@@ -1291,7 +1284,7 @@ void av_frac_init(AVFrac *f, INT64 val, INT64 num, INT64 den)
 }
 
 /* set f to (val + 0.5) */
-void av_frac_set(AVFrac *f, INT64 val)
+void av_frac_set(AVFrac *f, int64_t val)
 {
     f->val = val;
     f->num = f->den >> 1;
@@ -1303,9 +1296,9 @@ void av_frac_set(AVFrac *f, INT64 val)
  * @param f fractional number
  * @param incr increment, can be positive or negative
  */
-void av_frac_add(AVFrac *f, INT64 incr)
+void av_frac_add(AVFrac *f, int64_t incr)
 {
-    INT64 num, den;
+    int64_t num, den;
 
     num = f->num + incr;
     den = f->den;
@@ -1383,7 +1376,7 @@ int av_read_image(ByteIOContext *pb, const char *filename,
     int ret;
 
     if (!fmt) {
-        pd->filename = (char *)filename;
+        pd->filename = filename;
         pd->buf = buf;
         pos = url_ftell(pb);
         pd->buf_size = get_buffer(pb, buf, PROBE_BUF_SIZE);
