@@ -4,7 +4,7 @@
  * control the pace of reading.
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: input_ext-intf.h,v 1.85 2003/02/08 22:20:28 massiot Exp $
+ * $Id: input_ext-intf.h,v 1.87 2003/03/24 17:15:29 gbazin Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -33,6 +33,7 @@
 #define REQUESTED_MPEG         1
 #define REQUESTED_A52          2
 #define REQUESTED_LPCM         3
+#define REQUESTED_DTS          4
 #define REQUESTED_NOAUDIO    255
 
 #define OFFSETTOTIME_MAX_SIZE       10
@@ -221,7 +222,6 @@ struct stream_descriptor_t
                                                disc or network */
     vlc_bool_t              b_pace_control;    /* can we read when we want ? */
     vlc_bool_t              b_seekable;               /* can we do lseek() ? */
-    vlc_bool_t              b_connected;     /* does read() == 0 imply EOF ? */
 
     /* if (b_seekable) : */
     unsigned int            i_area_nb;
