@@ -2,7 +2,7 @@
  * http.c: HTTP access plug-in
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: http.c,v 1.23 2003/02/08 19:10:20 massiot Exp $
+ * $Id: http.c,v 1.32 2003/03/30 18:14:35 gbazin Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -26,7 +26,6 @@
  *****************************************************************************/
 #include <stdlib.h>
 #include <string.h>
-#include <sys/time.h>
 #include <vlc/vlc.h>
 #include <vlc/input.h>
 
@@ -43,8 +42,6 @@
 
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>
-#elif defined( _MSC_VER ) && defined( _WIN32 ) && !defined( UNDER_CE )
-#   include <io.h>
 #endif
 
 #if defined( UNDER_CE )
@@ -68,7 +65,6 @@
 static int  Open       ( vlc_object_t * );
 static void Close      ( vlc_object_t * );
 
-static int  SetProgram ( input_thread_t *, pgrm_descriptor_t * );
 static void Seek       ( input_thread_t *, off_t );
 static ssize_t Read    ( input_thread_t *, byte_t *, size_t );
 
@@ -87,10 +83,10 @@ static ssize_t Read    ( input_thread_t *, byte_t *, size_t );
     "value should be set in miliseconds units." )
 
 vlc_module_begin();
-    add_category_hint( N_("http"), NULL );
-    add_string( "http-proxy", NULL, NULL, PROXY_TEXT, PROXY_LONGTEXT );
-    add_integer( "http-caching", 4 * DEFAULT_PTS_DELAY / 1000, NULL, CACHING_TEXT, CACHING_LONGTEXT );
-    set_description( _("HTTP access module") );
+    add_category_hint( N_("http"), NULL, VLC_FALSE );
+    add_string( "http-proxy", NULL, NULL, PROXY_TEXT, PROXY_LONGTEXT, VLC_FALSE );
+    add_integer( "http-caching", 4 * DEFAULT_PTS_DELAY / 1000, NULL, CACHING_TEXT, CACHING_LONGTEXT, VLC_TRUE );
+    set_description( _("HTTP input") );
     set_capability( "access", 0 );
     add_shortcut( "http" );
     add_shortcut( "http4" );
@@ -342,7 +338,8 @@ static int HTTPConnect( input_thread_t * p_input, off_t i_tell )
             p_playlist->pp_items[p_playlist->i_index]->b_autodeletion
                                                                   = VLC_TRUE;
             playlist_Add( p_playlist, psz_value,
-                          PLAYLIST_APPEND | PLAYLIST_GO, PLAYLIST_END );
+                          PLAYLIST_INSERT | PLAYLIST_GO,
+                          p_playlist->i_index + 1 );
             vlc_object_release( p_playlist );
         }
 
@@ -606,14 +603,13 @@ static int Open( vlc_object_t *p_this )
                       psz_server_addr, i_server_port, psz_path );
 
     p_input->pf_read = Read;
-    p_input->pf_set_program = SetProgram;
+    p_input->pf_set_program = input_SetProgram;
     p_input->pf_set_area = NULL;
     p_input->pf_seek = Seek;
 
     vlc_mutex_lock( &p_input->stream.stream_lock );
     p_input->stream.b_pace_control = VLC_TRUE;
     p_input->stream.b_seekable = VLC_TRUE;
-    p_input->stream.b_connected = VLC_TRUE;
     p_input->stream.p_selected_area->i_tell = 0;
     p_input->stream.p_selected_area->i_size = 0;
     p_input->stream.i_method = INPUT_METHOD_NETWORK;
@@ -669,15 +665,6 @@ static void Close( vlc_object_t *p_this )
     free( p_access_data );
 }
 
-/*****************************************************************************
- * SetProgram: do nothing
- *****************************************************************************/
-static int SetProgram( input_thread_t * p_input,
-                       pgrm_descriptor_t * p_program )
-{
-    return VLC_SUCCESS;
-}
-
 /*****************************************************************************
  * Seek: close and re-open a connection at the right place
  *****************************************************************************/
@@ -694,13 +681,15 @@ static void Seek( input_thread_t * p_input, off_t i_pos )
 }
 
 /*****************************************************************************
- * Read: read on a file descriptor, checking b_die periodically
+ * Read: Read up to i_len bytes from the http connection and place in
+ * p_buffer. Return the actual number of bytes read
  *****************************************************************************/
 static ssize_t Read( input_thread_t * p_input, byte_t * p_buffer, size_t i_len )
 {
     input_socket_t * p_access_data = (input_socket_t *)p_input->p_access_data;
     struct timeval  timeout;
     fd_set          fds;
+    ssize_t         i_recv;
     int             i_ret;
 
     /* Initialize file descriptor set */
@@ -712,35 +701,40 @@ static ssize_t Read( input_thread_t * p_input, byte_t * p_buffer, size_t i_len )
     timeout.tv_usec = 500000;
 
     /* Find if some data is available */
-    i_ret = select( p_access_data->i_handle + 1, &fds,
-                    NULL, NULL, &timeout );
-
+    while( (i_ret = select( p_access_data->i_handle + 1, &fds,
+                            NULL, NULL, &timeout )) == 0
 #ifdef HAVE_ERRNO_H
-    if( i_ret == -1 && errno != EINTR )
+           || (i_ret < 0 && errno == EINTR)
+#endif
+           )
     {
-        msg_Err( p_input, "network select error (%s)", strerror(errno) );
+        FD_ZERO( &fds );
+        FD_SET( p_access_data->i_handle, &fds );
+        timeout.tv_sec = 0;
+        timeout.tv_usec = 500000;
+
+        if( p_input->b_die || p_input->b_error )
+        {
+            return 0;
+        }
     }
-#else
-    if( i_ret == -1 )
+
+    if( i_ret < 0 )
     {
         msg_Err( p_input, "network select error" );
+        return -1;
     }
-#endif
-    else if( i_ret > 0 )
+
+    i_recv = recv( p_access_data->i_handle, p_buffer, i_len, 0 );
+
+    if( i_recv < 0 )
     {
-        ssize_t i_recv = recv( p_access_data->i_handle, p_buffer, i_len, 0 );
-
-        if( i_recv < 0 )
-        {
 #ifdef HAVE_ERRNO_H
-            msg_Err( p_input, "recv failed (%s)", strerror(errno) );
+        msg_Err( p_input, "recv failed (%s)", strerror(errno) );
 #else
-            msg_Err( p_input, "recv failed" );
+        msg_Err( p_input, "recv failed" );
 #endif
-        }
-
-        return i_recv;
     }
 
-    return 0;
+    return i_recv;
 }
