@@ -2,7 +2,7 @@
  * slp.c: SLP access plugin
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: slp.c,v 1.8 2003/02/20 01:52:45 sigmunau Exp $
+ * $Id: slp.c,v 1.10 2003/03/30 18:14:35 gbazin Exp $
  *
  * Authors: Loc Minier <lool@videolan.org>
  *
@@ -75,7 +75,7 @@ static int  Demux ( input_thread_t * );
                       "default locale; it is used in all SLP queries"
 
 vlc_module_begin();
-    set_description( _("SLP access module") );
+    set_description( _("SLP input") );
     add_category_hint( N_("slp"), NULL, VLC_TRUE );
     add_string( "slp-attrids", "", NULL, ATTRIDS_TEXT, ATTRIDS_LONGTEXT, VLC_TRUE );
     add_string( "slp-scopelist", "", NULL, SCOPELIST_TEXT,
@@ -340,7 +340,6 @@ static int Open( vlc_object_t * p_this )
     vlc_mutex_lock( &p_input->stream.stream_lock );
     p_input->stream.b_pace_control = VLC_FALSE;
     p_input->stream.b_seekable = VLC_FALSE;
-    p_input->stream.b_connected = VLC_TRUE;
     p_input->stream.p_selected_area->i_tell = 0;
     p_input->stream.p_selected_area->i_size = 0;
     p_input->stream.i_method = INPUT_METHOD_SLP;
