@@ -2,7 +2,7 @@
  * file.c : audio output which writes the samples to a file
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: file.c,v 1.15 2003/02/08 19:10:20 massiot Exp $
+ * $Id: file.c,v 1.20 2003/03/30 18:14:36 gbazin Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Gildas Bazin <gbazin@netcourrier.com>
@@ -101,17 +101,17 @@ static int format_int[] = { VLC_FOURCC('u','8',' ',' '),
                             VLC_FOURCC('f','l','3','2'),
                             VLC_FOURCC('s','p','i','f') };
 
-#define PATH_TEXT N_("path of the output file")
-#define PATH_LONGTEXT N_("By default samples.raw")
+#define FILE_TEXT N_("output file")
+#define FILE_LONGTEXT N_("file to which the audio samples will be written to")
 
 vlc_module_begin();
-    add_category_hint( N_("Audio"), NULL );
+    add_category_hint( N_("Audio"), NULL, VLC_FALSE );
     add_string_from_list( "audiofile-format", "s16", format_list, NULL,
-                          FORMAT_TEXT, FORMAT_LONGTEXT );
-    add_string( "audiofile-path", "audiofile.wav", NULL, PATH_TEXT,
-                PATH_LONGTEXT );
-    add_bool( "audiofile-wav", 1, NULL, WAV_TEXT, WAV_LONGTEXT );
-    set_description( _("file audio output module") );
+                          FORMAT_TEXT, FORMAT_LONGTEXT, VLC_TRUE );
+    add_file( "audiofile", "audiofile.wav", NULL, FILE_TEXT,
+              FILE_LONGTEXT, VLC_FALSE );
+    add_bool( "audiofile-wav", 1, NULL, WAV_TEXT, WAV_LONGTEXT, VLC_TRUE );
+    set_description( N_("file audio output") );
     set_capability( "audio output", 0 );
     add_shortcut( "file" );
     add_shortcut( "audiofile" );
