@@ -2,7 +2,7 @@
  * ffmpeg_vdec.h: video decoder using ffmpeg library
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: ffmpeg.h,v 1.12 2003/02/18 19:42:57 gbazin Exp $
+ * $Id: ffmpeg.h,v 1.15 2003/03/24 13:50:55 hartman Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  * 
@@ -108,9 +108,21 @@ int E_( GetPESData )( u8 *p_buf, int i_max, pes_packet_t *p_pes );
 /* AngelPotion stuff */
 #define FOURCC_AP41         VLC_FOURCC('A','P','4','1')
 
-/* ?? */
+/* 3ivx doctered divx files */
+#define FOURCC_3IVD         VLC_FOURCC('3','I','V','D')
+#define FOURCC_3ivd         VLC_FOURCC('3','i','v','d')
+
+/* 3ivx delta 3.5 Unsupported */ 
 #define FOURCC_3IV1         VLC_FOURCC('3','I','V','1')
+#define FOURCC_3iv1         VLC_FOURCC('3','i','v','1')
+
+/* 3ivx delta 4 */
 #define FOURCC_3IV2         VLC_FOURCC('3','I','V','2')
+#define FOURCC_3iv2         VLC_FOURCC('3','i','v','2')
+
+/* who knows? */
+#define FOURCC_3VID         VLC_FOURCC('3','V','I','D')
+#define FOURCC_3vid         VLC_FOURCC('3','v','i','d')
 
 /* H263 and H263i */        
 #define FOURCC_H263         VLC_FOURCC('H','2','6','3')
@@ -137,6 +149,7 @@ int E_( GetPESData )( u8 *p_buf, int i_max, pes_packet_t *p_pes );
 /* wmv */
 #define FOURCC_WMV1         VLC_FOURCC('W','M','V','1')
 #define FOURCC_WMV2         VLC_FOURCC('W','M','V','2')
+#define FOURCC_MSS1         VLC_FOURCC('M','S','S','1')
 
 #define FOURCC_dvsl         VLC_FOURCC('d','v','s','l')
 #define FOURCC_dvsd         VLC_FOURCC('d','v','s','d')
@@ -154,3 +167,6 @@ int E_( GetPESData )( u8 *p_buf, int i_max, pes_packet_t *p_pes );
 #define FOURCC_wma2         VLC_FOURCC('w','m','a','2')
 #define FOURCC_dvau         VLC_FOURCC('d','v','a','u')
 
+#define FOURCC_MAC3         VLC_FOURCC('M','A','C','3')
+#define FOURCC_MAC6         VLC_FOURCC('M','A','C','6')
+
