@@ -2,7 +2,7 @@
  * m3u.c: a meta demux to parse pls, m3u and asx playlists
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: m3u.c,v 1.15 2003/03/06 15:30:42 sigmunau Exp $
+ * $Id: m3u.c,v 1.18 2003/04/06 20:08:11 sigmunau Exp $
  *
  * Authors: Sigmund Augdal <sigmunau@idi.ntnu.no>
  *          Gildas Bazin <gbazin@netcourrier.com>
@@ -63,7 +63,7 @@ static int  Demux ( input_thread_t * );
  *****************************************************************************/
 vlc_module_begin();
     set_description( _("playlist metademux") );
-    set_capability( "demux", 10 );
+    set_capability( "demux", 180 );
     set_callbacks( Activate, Deactivate );
     add_shortcut( "m3u" );
     add_shortcut( "asx" );
@@ -79,7 +79,8 @@ static int Activate( vlc_object_t * p_this )
     input_thread_t *p_input = (input_thread_t *)p_this;
     char           *psz_ext;
     demux_sys_t    *p_m3u;
-    int            i_type = 0;
+    int             i_type = 0;
+    int             i_type2 = 0;
 
     /* Initialize access plug-in structures. */
     if( p_input->i_mtu == 0 )
@@ -91,37 +92,35 @@ static int Activate( vlc_object_t * p_this )
     p_input->pf_demux = Demux;
     p_input->pf_rewind = NULL;
 
-    /* Check for m3u/asx file extension */
+    /* Check for m3u/asx file extension or if the demux has been forced */
     psz_ext = strrchr ( p_input->psz_name, '.' );
-    if( psz_ext )
+
+    if( ( psz_ext && !strcasecmp( psz_ext, ".m3u") ) ||
+        ( p_input->psz_demux && !strcmp(p_input->psz_demux, "m3u") ) )
     {
-        if( !strcasecmp( psz_ext, ".m3u") ||
-            ( p_input->psz_demux && !strncmp(p_input->psz_demux, "m3u", 3) ) )
-        {
-            i_type = TYPE_M3U;
-        }
-        else if( !strcasecmp( psz_ext, ".asx") ||
-                 ( p_input->psz_demux && !strncmp(p_input->psz_demux, "asx", 3) ) )
-        {
-            i_type = TYPE_ASX;
-        }
-        else if( !strcasecmp( psz_ext, ".html") ||
-                 ( p_input->psz_demux && !strncmp(p_input->psz_demux, "html", 4) ) )
-        {
-            i_type = TYPE_HTML;
-        }
-        else if( !strcasecmp( psz_ext, ".pls") ||
-                 ( p_input->psz_demux && !strncmp(p_input->psz_demux, "pls", 3) ) )
-        {
-            i_type = TYPE_PLS;
-        }
+        i_type = TYPE_M3U;
+    }
+    else if( ( psz_ext && !strcasecmp( psz_ext, ".asx") ) ||
+             ( p_input->psz_demux && !strcmp(p_input->psz_demux, "asx") ) )
+    {
+        i_type = TYPE_ASX;
+    }
+    else if( ( psz_ext && !strcasecmp( psz_ext, ".html") ) ||
+             ( p_input->psz_demux && !strcmp(p_input->psz_demux, "html") ) )
+    {
+        i_type = TYPE_HTML;
+    }
+    else if( ( psz_ext && !strcasecmp( psz_ext, ".pls") ) ||
+             ( p_input->psz_demux && !strcmp(p_input->psz_demux, "pls") ) )
+    {
+        i_type = TYPE_PLS;
     }
 
     /* we had no luck looking at the file extention, so we have a look
      * at the content. This is useful for .asp, .php and similar files
      * that are actually html. Also useful for som asx files that have
      * another extention */
-    if( !i_type )
+    if( i_type != TYPE_M3U )
     {
         byte_t *p_peek;
         int i_size = input_Peek( p_input, &p_peek, MAX_LINE );
@@ -137,22 +136,35 @@ static int Activate( vlc_object_t * p_this )
             }
             if ( !i_size )
             {
-                return -1;
+                ;
             }
             else if ( !strncasecmp( p_peek, "[playlist]", sizeof("[playlist]") -1 ) )
             {
-                i_type = TYPE_PLS;
+                i_type2 = TYPE_PLS;
             }
             else if ( !strncasecmp( p_peek, "<html>", sizeof("<html>") -1 ) )
             {
-                i_type = TYPE_HTML;
+                i_type2 = TYPE_HTML;
             }
             else if ( !strncasecmp( p_peek, "<asx", sizeof("<asx") -1 ) )
             {
-                i_type = TYPE_ASX;
+                i_type2 = TYPE_ASX;
             }
         }
     }
+    if ( !i_type && !i_type2 )
+    {
+        return -1;
+    }
+    if ( i_type  && !i_type2 )
+    {
+        i_type = TYPE_M3U;
+    }
+    else
+    {
+        i_type = i_type2;
+    }
+
     /* Allocate p_m3u */
     if( !( p_m3u = malloc( sizeof( demux_sys_t ) ) ) )
     {
@@ -237,6 +249,10 @@ static int ProcessLine ( input_thread_t *p_input , demux_sys_t *p_m3u
         while( *psz_bol &&
                strncasecmp( psz_bol, "mms://",
                             sizeof("mms://") - 1 ) &&
+               strncasecmp( psz_bol, "mmsu://",
+                            sizeof("mmsu://") - 1 ) &&
+               strncasecmp( psz_bol, "mmst://",
+                            sizeof("mmst://") - 1 ) &&
                strncasecmp( psz_bol, "http://",
                             sizeof("http://") - 1 ) &&
                strncasecmp( psz_bol, "file://",
