@@ -2,7 +2,7 @@
  * mpeg_ts.c : Transport Stream input module for vlc
  *****************************************************************************
  * Copyright (C) 2000-2001 VideoLAN
- * $Id: ts.c,v 1.17 2003/02/12 13:42:43 massiot Exp $
+ * $Id: ts.c,v 1.20 2003/03/18 23:59:07 massiot Exp $
  *
  * Authors: Henri Fallon <henri@via.ecp.fr>
  *          Johan Bilien <jobi@via.ecp.fr>
@@ -115,10 +115,10 @@ vlc_module_begin();
     set_capability( "demux", 170 );
     add_shortcut( "ts_dvbpsi" );
 #endif
-    add_category_hint( N_("Miscellaneous"), NULL );
+    add_category_hint( N_("Miscellaneous"), NULL, VLC_TRUE );
     add_bool( "vls-backwards-compat", 0, NULL,
-              VLS_BACKWARDS_COMPAT_TEXT, VLS_BACKWARDS_COMPAT_LONGTEXT );
-    add_bool( "buggy-psi", 0, NULL, BUGGY_PSI_TEXT, BUGGY_PSI_LONGTEXT );
+              VLS_BACKWARDS_COMPAT_TEXT, VLS_BACKWARDS_COMPAT_LONGTEXT, VLC_TRUE );
+    add_bool( "buggy-psi", 0, NULL, BUGGY_PSI_TEXT, BUGGY_PSI_LONGTEXT, VLC_TRUE );
     set_callbacks( Activate, Deactivate );
 vlc_module_end();
 
@@ -641,6 +641,7 @@ static void TSDecodePMT( input_thread_t * p_input, es_descriptor_t * p_es )
                     {
                         case MPEG1_VIDEO_ES:
                         case MPEG2_VIDEO_ES:
+                        case MPEG2_MOTO_VIDEO_ES:
                             /* This isn't real, but we don't actually use
                              * it. */
                             p_new_es->i_stream_id = 0xE0;
@@ -1295,6 +1296,7 @@ static void TS_DVBPSI_HandlePMT( input_thread_t * p_input,
             {
                 case MPEG1_VIDEO_ES:
                 case MPEG2_VIDEO_ES:
+                case MPEG2_MOTO_VIDEO_ES:
                     p_new_es->i_fourcc = VLC_FOURCC('m','p','g','v');
                     p_new_es->i_cat = VIDEO_ES;
                     break;
@@ -1360,7 +1362,11 @@ static void TS_DVBPSI_HandlePMT( input_thread_t * p_input,
                     p_new_es->i_cat = AUDIO_ES;
                     p_new_es->i_stream_id = 0xfa;
                     break;
-
+                case MSCODEC_VIDEO_ES:
+                    p_new_es->i_fourcc = VLC_FOURCC(0,0,0,0);   // fixed later
+                    p_new_es->i_cat = VIDEO_ES;
+                    p_new_es->i_stream_id = 0xa0;
+                    break;
                 default:
                     p_new_es->i_fourcc = 0;
                     p_new_es->i_cat = UNKNOWN_ES;
@@ -1517,6 +1523,50 @@ static void TS_DVBPSI_HandlePMT( input_thread_t * p_input,
                 }
 
             }
+            else if( p_es->i_type == MSCODEC_VIDEO_ES )
+            {
+                /* crapy ms codec stream, search private descriptor */
+                dvbpsi_descriptor_t *p_dr = p_es->p_first_descriptor;
+                es_ts_data_t        *p_es_demux =
+                                        (es_ts_data_t*)p_new_es->p_demux_data;
+
+                while( p_dr && ( p_dr->i_tag != 0xa0 ) )
+                    p_dr = p_dr->p_next;
+                if( p_dr && p_dr->i_length >= 8 )
+                {
+                    int i_size;
+                    int i_bih_size;
+                    BITMAPINFOHEADER *p_bih;
+
+                    p_new_es->i_fourcc = ( p_dr->p_data[0] << 24 )| ( p_dr->p_data[1] << 16 )|
+                                         ( p_dr->p_data[2] <<  8 )| ( p_dr->p_data[3] );
+
+                    i_bih_size = ( ( p_dr->p_data[8] << 8)|( p_dr->p_data[9] ) );
+                    i_size = sizeof( BITMAPINFOHEADER ) + i_bih_size;
+
+                    p_new_es->p_bitmapinfoheader = (void*)p_bih = malloc( i_size );
+                    p_bih->biSize = i_size;
+                    p_bih->biWidth = ( p_dr->p_data[4] << 8 )|p_dr->p_data[5];
+                    p_bih->biHeight = ( p_dr->p_data[6] << 8 )|p_dr->p_data[7];
+                    p_bih->biPlanes = 1;
+                    p_bih->biBitCount = 0;
+                    p_bih->biCompression = 0;
+                    p_bih->biSizeImage = 0;
+                    p_bih->biXPelsPerMeter = 0;
+                    p_bih->biYPelsPerMeter = 0;
+                    p_bih->biClrUsed = 0;
+                    p_bih->biClrImportant = 0;
+                    memcpy( &p_bih[1],
+                            &p_dr->p_data[10],
+                            i_bih_size );
+                }
+                else
+                {
+                    msg_Warn( p_input, "private ms-codec stream without bih private sl_descriptor" );
+                    p_new_es->i_fourcc = 0;
+                    p_new_es->i_cat = UNKNOWN_ES;
+                }
+            }
 
             if(    ( p_new_es->i_cat == AUDIO_ES )
                 || (p_new_es->i_cat == SPU_ES ) )
