@@ -2,7 +2,7 @@
  * sub.c
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: sub.c,v 1.6 2003/02/20 01:52:46 sigmunau Exp $
+ * $Id: sub.c,v 1.11 2003/03/30 14:24:20 gbazin Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -47,14 +47,11 @@ static int  sub_demux( subtitle_demux_t *p_sub, mtime_t i_maxdate );
 static int  sub_seek ( subtitle_demux_t *p_sub, mtime_t i_date );
 static void sub_close( subtitle_demux_t *p_sub );
 
-static int  sub_MicroDvdRead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
-static int  sub_SubRipRead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
-static int  sub_SSA1Read( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
-static int  sub_SSA2_4Read( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
-
 static void sub_fix( subtitle_demux_t *p_sub );
 
-static char *ppsz_sub_type[] = { "microdvd", "subrip", "ssa1", "ssa2-4", NULL };
+static char *ppsz_sub_type[] = { "microdvd", "subrip", "ssa1", "ssa2-4", "vplayer", "sami", NULL };
+
+
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
@@ -63,15 +60,15 @@ static char *ppsz_sub_type[] = { "microdvd", "subrip", "ssa1", "ssa2-4", NULL };
     "Override frames per second. " \
     "It will work only with MicroDVD"
 #define SUB_TYPE_LONGTEXT \
-    "One from \"microdvd\", \"subrip\", \"ssa1\", \"ssa2-4\" " \
+    "One from \"microdvd\", \"subrip\", \"ssa1\", \"ssa2-4\", \"vplayer\" \"sami\"" \
     "(nothing for autodetection, it should always work)"
 
 vlc_module_begin();
     set_description( _("text subtitle demux") );
     set_capability( "subtitle demux", 12 );
     add_category_hint( "subtitle", NULL, VLC_TRUE );
-        add_string( "sub-file", NULL, NULL,
-                    "subtitle file name", "subtitle file name", VLC_TRUE );
+        add_file( "sub-file", NULL, NULL,
+                  "subtitle file name", "subtitle file name", VLC_TRUE );
         add_float( "sub-fps", 0.0, NULL,
                    "override frames per second",
                    SUB_FPS_LONGTEXT, VLC_TRUE );
@@ -100,6 +97,129 @@ static int Open ( vlc_object_t *p_this )
 }
 #define MAX_TRY     256
 #define MAX_LINE    2048
+
+#define FREE( p ) if( p ) { free( p); (p) = NULL; }
+
+typedef struct
+{
+    int     i_line_count;
+    int     i_line;
+    char    **line;
+} text_t;
+
+static int  text_load( text_t *txt, char *psz_name )
+{
+    FILE *f;
+    int   i_line_max;
+
+    /* init txt */
+    i_line_max          = 100;
+    txt->i_line_count   = 0;
+    txt->i_line         = 0;
+    txt->line           = calloc( i_line_max, sizeof( char * ) );
+
+    /* open file */
+    if( !( f = fopen( psz_name, "rb" ) ) )
+    {
+        return VLC_EGENERIC;
+    }
+
+    /* load the complete file */
+    for( ;; )
+    {
+        char buffer[8096];
+        char *p;
+
+        if( fgets( buffer, 8096, f ) <= 0)
+        {
+            break;
+        }
+        while( ( p = strchr( buffer, '\r' ) ) )
+        {
+            *p = '\0';
+        }
+        while( ( p = strchr( buffer, '\n' ) ) )
+        {
+            *p = '\0';
+        }
+
+        txt->line[txt->i_line_count++] = strdup( buffer );
+
+        if( txt->i_line_count >= i_line_max )
+        {
+            i_line_max += 100;
+            txt->line = realloc( txt->line, i_line_max * sizeof( char*) );
+        }
+    }
+
+    fclose( f );
+
+    if( txt->i_line_count <= 0 )
+    {
+        FREE( txt->line );
+        return( VLC_EGENERIC );
+    }
+
+    return( VLC_SUCCESS );
+}
+static void text_unload( text_t *txt )
+{
+    int i;
+
+    for( i = 0; i < txt->i_line_count; i++ )
+    {
+        FREE( txt->line[i] );
+    }
+    FREE( txt->line );
+    txt->i_line       = 0;
+    txt->i_line_count = 0;
+}
+
+static char *text_get_line( text_t *txt )
+{
+    if( txt->i_line >= txt->i_line_count )
+    {
+        return( NULL );
+    }
+
+    return( txt->line[txt->i_line++] );
+}
+static void text_previous_line( text_t *txt )
+{
+    if( txt->i_line > 0 )
+    {
+        txt->i_line--;
+    }
+}
+static void text_rewind( text_t *txt )
+{
+    txt->i_line = 0;
+}
+
+static int  sub_MicroDvdRead( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
+static int  sub_SubRipRead  ( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
+static int  sub_SSA1Read    ( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
+static int  sub_SSA2_4Read  ( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
+static int  sub_Vplayer     ( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
+static int  sub_Sami        ( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
+
+static struct
+{
+    char *psz_type_name;
+    int  i_type;
+    char *psz_name;
+    int  (*pf_read_subtitle)    ( text_t *, subtitle_t*, mtime_t );
+} sub_read_subtitle_function [] =
+{
+    { "microdvd",   SUB_TYPE_MICRODVD,  "MicroDVD", sub_MicroDvdRead },
+    { "subrip",     SUB_TYPE_SUBRIP,    "SubRIP",   sub_SubRipRead },
+    { "ssa1",       SUB_TYPE_SSA1,      "SSA-1",    sub_SSA1Read },
+    { "ssa2-4",     SUB_TYPE_SSA2_4,    "SSA-2/3/4",sub_SSA2_4Read },
+    { "vplayer",    SUB_TYPE_VPLAYER,   "VPlayer",  sub_Vplayer },
+    { "sami",       SUB_TYPE_SAMI,      "SAMI",     sub_Sami },
+    { NULL,         SUB_TYPE_UNKNOWN,   "Unknow",   NULL }
+};
+
 /*****************************************************************************
  * sub_open: Open a subtitle file and add subtitle ES
  *****************************************************************************/
@@ -108,13 +228,13 @@ static int  sub_open ( subtitle_demux_t *p_sub,
                        char     *psz_name,
                        mtime_t i_microsecperframe )
 {
-    FILE *p_file;
+    text_t  txt;
+
+    int     i;
     char    *psz_file_type;
-    char    buffer[MAX_LINE + 1];
-    int     i_try;
     int     i_sub_type;
     int     i_max;
-    int (*pf_read_subtitle)( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_microsecperframe ) = NULL;
+    int (*pf_read_subtitle)( text_t *, subtitle_t *, mtime_t ) = NULL;
 
     p_sub->i_sub_type = SUB_TYPE_UNKNOWN;
     p_sub->p_es = NULL;
@@ -135,6 +255,17 @@ static int  sub_open ( subtitle_demux_t *p_sub,
         psz_name = strdup( psz_name );
     }
 
+    /* *** load the file *** */
+    if( text_load( &txt, psz_name ) )
+    {
+        msg_Err( p_sub, "cannot open `%s' subtitle file", psz_name );
+        free( psz_name );
+        return VLC_EGENERIC;
+    }
+    msg_Dbg( p_sub, "opened `%s'", psz_name );
+    free( psz_name );
+
+
     if(  config_GetFloat( p_sub, "sub-fps" ) >= 1.0 )
     {
         i_microsecperframe = (mtime_t)( (float)1000000 /
@@ -145,67 +276,60 @@ static int  sub_open ( subtitle_demux_t *p_sub,
         i_microsecperframe = 40000; /* default: 25fps */
     }
 
-    /* *** Open the file *** */
-    if( !( p_file = fopen( psz_name, "rb" ) ) )
-    {
-        msg_Err( p_sub, "cannot open `%s' subtitle file", psz_name );
-        free( psz_name );
-        return VLC_EGENERIC;
-    }
-    else
-    {
-        msg_Dbg( p_sub, "opened `%s'", psz_name );
-        free( psz_name );
-    }
-
     psz_file_type = config_GetPsz( p_sub, "sub-type" );
     if( psz_file_type && *psz_file_type)
     {
-        if( !strcmp( psz_file_type, "microdvd" ) )
+        int i;
+
+        for( i = 0; ; i++ )
         {
-            i_sub_type = SUB_TYPE_MICRODVD;
-        }
-        else if( !strcmp( psz_file_type, "subrip" ) )
-        {
-            i_sub_type = SUB_TYPE_SUBRIP;
-        }
-        else if( !strcmp( psz_file_type, "ssa1" ) )
-        {
-            i_sub_type = SUB_TYPE_SSA1;
-        }
-        else if( !strcmp( psz_file_type, "ssa2-4" ) )
-        {
-            i_sub_type = SUB_TYPE_SSA2_4;
-        }
-        else
-        {
-            i_sub_type = SUB_TYPE_UNKNOWN;
+            if( sub_read_subtitle_function[i].psz_type_name == NULL )
+            {
+                i_sub_type = SUB_TYPE_UNKNOWN;
+                break;
+            }
+            if( !strcmp( sub_read_subtitle_function[i].psz_type_name,
+                         psz_file_type ) )
+            {
+                i_sub_type = sub_read_subtitle_function[i].i_type;
+                break;
+            }
         }
     }
     else
     {
         i_sub_type = SUB_TYPE_UNKNOWN;
     }
+    FREE( psz_file_type );
 
     /* *** Now try to autodetect subtitle format *** */
     if( i_sub_type == SUB_TYPE_UNKNOWN )
     {
+        int     i_try;
+        char    *s;
+
         msg_Dbg( p_input, "trying to autodetect file format" );
         for( i_try = 0; i_try < MAX_TRY; i_try++ )
         {
             int i_dummy;
-            if( fgets( buffer, MAX_LINE, p_file ) <= 0 )
+
+            if( ( s = text_get_line( &txt ) ) == NULL )
             {
                 break;
             }
 
-            if( sscanf( buffer, "{%d}{%d}", &i_dummy, &i_dummy ) == 2 ||
-                sscanf( buffer, "{%d}{}", &i_dummy ) == 1)
+            if( strstr( s, "<SAMI>" ) )
+            {
+                i_sub_type = SUB_TYPE_SAMI;
+                break;
+            }
+            else if( sscanf( s, "{%d}{%d}", &i_dummy, &i_dummy ) == 2 ||
+                     sscanf( s, "{%d}{}", &i_dummy ) == 1)
             {
                 i_sub_type = SUB_TYPE_MICRODVD;
                 break;
             }
-            else if( sscanf( buffer,
+            else if( sscanf( s,
                              "%d:%d:%d,%d --> %d:%d:%d,%d",
                              &i_dummy,&i_dummy,&i_dummy,&i_dummy,
                              &i_dummy,&i_dummy,&i_dummy,&i_dummy ) == 8 )
@@ -213,7 +337,7 @@ static int  sub_open ( subtitle_demux_t *p_sub,
                 i_sub_type = SUB_TYPE_SUBRIP;
                 break;
             }
-            else if( sscanf( buffer,
+            else if( sscanf( s,
                              "!: This is a Sub Station Alpha v%d.x script.",
                              &i_dummy ) == 1)
             {
@@ -225,46 +349,48 @@ static int  sub_open ( subtitle_demux_t *p_sub,
                 {
                     i_sub_type = SUB_TYPE_SSA2_4; // I hop this will work
                 }
+                break;
             }
-            else if( !strcmp( buffer,
-                              "Dialogue: Marked" ) )
+            else if( strstr( s, "This is a Sub Station Alpha v4 script" ) )
+            {
+                i_sub_type = SUB_TYPE_SSA2_4; // I hop this will work
+            }
+            else if( !strncmp( s, "Dialogue: Marked", 16  ) )
             {
                 i_sub_type = SUB_TYPE_SSA2_4; // could be wrong
+                break;
+            }
+            else if( sscanf( s, "%d:%d:%d:", &i_dummy, &i_dummy, &i_dummy ) == 3 ||
+                     sscanf( s, "%d:%d:%d ", &i_dummy, &i_dummy, &i_dummy ) == 3 )
+            {
+                i_sub_type = SUB_TYPE_VPLAYER;
+                break;
             }
         }
+
+        text_rewind( &txt );
     }
 
     /* *** Load this file in memory *** */
-    switch( i_sub_type )
+    for( i = 0; ; i++ )
     {
-        case SUB_TYPE_MICRODVD:
-            msg_Dbg( p_input, "detected MicroDVD format" );
-            pf_read_subtitle = sub_MicroDvdRead;
-            break;
-        case SUB_TYPE_SUBRIP:
-            msg_Dbg( p_input, "detected SubRIP format" );
-            pf_read_subtitle = sub_SubRipRead;
-            break;
-        case SUB_TYPE_SSA1:
-            msg_Dbg( p_input, "detected SSAv1 Script format" );
-            pf_read_subtitle = sub_SSA1Read;
-            break;
-        case SUB_TYPE_SSA2_4:
-            msg_Dbg( p_input, "detected SSAv2-4 Script format" );
-            pf_read_subtitle = sub_SSA2_4Read;
-            break;
-        default:
-            msg_Err( p_sub, "unknown subtitile file" );
-            fclose( p_file );
+        if( sub_read_subtitle_function[i].i_type == SUB_TYPE_UNKNOWN )
+        {
+            msg_Dbg( p_input, "unknown subtitile file" );
+            text_unload( &txt );
             return VLC_EGENERIC;
+        }
+
+        if( sub_read_subtitle_function[i].i_type == i_sub_type )
+        {
+            msg_Dbg( p_input,
+                    "detected %s format",
+                    sub_read_subtitle_function[i].psz_name );
+            pf_read_subtitle = sub_read_subtitle_function[i].pf_read_subtitle;
+            break;
+        }
     }
 
-    if( fseek( p_file, 0L, SEEK_SET ) < 0 )
-    {
-        msg_Err( p_input, "cannot read file from begining" );
-        fclose( p_file );
-        return VLC_EGENERIC;
-    }
     for( i_max = 0;; )
     {
         if( p_sub->i_subtitles >= i_max )
@@ -280,7 +406,7 @@ static int  sub_open ( subtitle_demux_t *p_sub,
                 p_sub->subtitle = malloc( sizeof( subtitle_t ) * i_max );
             }
         }
-        if( pf_read_subtitle( p_file,
+        if( pf_read_subtitle( &txt,
                               p_sub->subtitle + p_sub->i_subtitles,
                               i_microsecperframe ) < 0 )
         {
@@ -291,7 +417,7 @@ static int  sub_open ( subtitle_demux_t *p_sub,
     msg_Dbg( p_sub, "loaded %d subtitles", p_sub->i_subtitles );
 
     /* *** Close the file *** */
-    fclose( p_file );
+    text_unload( &txt );
 
     /* *** fix subtitle (order and time) *** */
     p_sub->i_subtitle = 0;  // will be modified by sub_fix
@@ -495,7 +621,7 @@ static void  sub_fix( subtitle_demux_t *p_sub )
 /*****************************************************************************
  * Specific Subtitle function
  *****************************************************************************/
-static int  sub_MicroDvdRead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_microsecperframe)
+static int  sub_MicroDvdRead( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe)
 {
     /*
      * each line:
@@ -503,25 +629,25 @@ static int  sub_MicroDvdRead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_mi
      * where n1 and n2 are the video frame number...
      *
      */
-    char buffer[MAX_LINE + 1];
+    char *s;
+
     char buffer_text[MAX_LINE + 1];
     uint32_t    i_start;
     uint32_t    i_stop;
-    int         i;
+    unsigned int i;
 
     for( ;; )
     {
-        if( fgets( buffer, MAX_LINE, p_file ) <= 0)
+        if( ( s = text_get_line( txt ) ) == NULL )
         {
-            return( -1 );
+            return( VLC_EGENERIC );
         }
         i_start = 0;
         i_stop  = 0;
 
-        buffer[MAX_LINE] = '\0';
         memset( buffer_text, '\0', MAX_LINE );
-        if( sscanf( buffer, "{%d}{}%[^\r\n]", &i_start, buffer_text ) == 2 ||
-            sscanf( buffer, "{%d}{%d}%[^\r\n]", &i_start, &i_stop, buffer_text ) == 3)
+        if( sscanf( s, "{%d}{}%[^\r\n]", &i_start, buffer_text ) == 2 ||
+            sscanf( s, "{%d}{%d}%[^\r\n]", &i_start, &i_stop, buffer_text ) == 3)
         {
             break;
         }
@@ -540,7 +666,7 @@ static int  sub_MicroDvdRead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_mi
     return( 0 );
 }
 
-static int  sub_SubRipRead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_microsecperframe )
+static int  sub_SubRipRead( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe )
 {
     /*
      * n
@@ -551,7 +677,7 @@ static int  sub_SubRipRead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_micr
      * [empty line]
      *
      */
-    char buffer[MAX_LINE + 1];
+    char *s;
     char buffer_text[ 10 * MAX_LINE];
     int  i_buffer_text;
     mtime_t     i_start;
@@ -560,11 +686,11 @@ static int  sub_SubRipRead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_micr
     for( ;; )
     {
         int h1, m1, s1, d1, h2, m2, s2, d2;
-        if( fgets( buffer, MAX_LINE, p_file ) <= 0)
+        if( ( s = text_get_line( txt ) ) == NULL )
         {
-            return( -1 );
+            return( VLC_EGENERIC );
         }
-        if( sscanf( buffer,
+        if( sscanf( s,
                     "%d:%d:%d,%d --> %d:%d:%d,%d",
                     &h1, &m1, &s1, &d1,
                     &h2, &m2, &s2, &d2 ) == 8 )
@@ -583,13 +709,13 @@ static int  sub_SubRipRead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_micr
             for( i_buffer_text = 0;; )
             {
                 int i_len;
-                if( fgets( buffer, MAX_LINE, p_file ) <= 0 )
+                if( ( s = text_get_line( txt ) ) == NULL )
                 {
-                    return( -1 );
+                    return( VLC_EGENERIC );
                 }
-                buffer[MAX_LINE] = '\0'; // just in case
-                i_len = strlen( buffer );
-                if( buffer[0] == '\r' || buffer[0] == '\n' || i_len <= 1 )
+
+                i_len = strlen( s );
+                if( i_len <= 1 )
                 {
                     // empty line -> end of this subtitle
                     buffer_text[__MAX( i_buffer_text - 1, 0 )] = '\0';
@@ -603,7 +729,7 @@ static int  sub_SubRipRead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_micr
                     if( i_buffer_text + i_len + 1 < 10 * MAX_LINE )
                     {
                         memcpy( buffer_text + i_buffer_text,
-                                buffer,
+                                s,
                                 i_len );
                         i_buffer_text += i_len;
 
@@ -617,10 +743,10 @@ static int  sub_SubRipRead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_micr
 }
 
 
-static int  sub_SSARead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_microsecperframe, int i_comma_count )
+static int  sub_SSARead( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe, int i_comma_count )
 {
-    char buffer[MAX_LINE + 1];
     char buffer_text[ 10 * MAX_LINE];
+    char *s;
     char *p_buffer_text;
     mtime_t     i_start;
     mtime_t     i_stop;
@@ -631,11 +757,12 @@ static int  sub_SSARead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_microse
     {
         int h1, m1, s1, c1, h2, m2, s2, c2;
         int i_dummy;
-        if( fgets( buffer, MAX_LINE, p_file ) <= 0 )
+
+        if( ( s = text_get_line( txt ) ) == NULL )
         {
-            return( -1 );
+            return( VLC_EGENERIC );
         }
-        if( sscanf( buffer,
+        if( sscanf( s,
                     "Dialogue: Marked=%d,%d:%d:%d.%d,%d:%d:%d.%d,%[^\r\n]",
                     &i_dummy,
                     &h1, &m1, &s1, &c1,
@@ -695,12 +822,184 @@ static int  sub_SSARead( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_microse
     }
 }
 
-static int  sub_SSA1Read( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_microsecperframe )
+static int  sub_SSA1Read( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe )
 {
-    return( sub_SSARead( p_file, p_subtitle, i_microsecperframe, 8 ) );
+    return( sub_SSARead( txt, p_subtitle, i_microsecperframe, 8 ) );
 }
-static int  sub_SSA2_4Read( FILE *p_file, subtitle_t *p_subtitle, mtime_t i_microsecperframe )
+static int  sub_SSA2_4Read( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe )
 {
-    return( sub_SSARead( p_file, p_subtitle, i_microsecperframe, 9 ) );
+    return( sub_SSARead( txt, p_subtitle, i_microsecperframe, 9 ) );
+}
+
+static int  sub_Vplayer( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe)
+{
+    /*
+     * each line:
+     *  h:m:s:Line1|Line2|Line3....
+     *  or
+     *  h:m:s Line1|Line2|Line3....
+     * where n1 and n2 are the video frame number...
+     *
+     */
+    char *p;
+    char buffer_text[MAX_LINE + 1];
+    mtime_t    i_start;
+    unsigned int i;
+
+    for( ;; )
+    {
+        int h, m, s;
+        char c;
+
+        if( ( p = text_get_line( txt ) ) == NULL )
+        {
+            return( VLC_EGENERIC );
+        }
+
+        i_start = 0;
+
+        memset( buffer_text, '\0', MAX_LINE );
+        if( sscanf( p, "%d:%d:%d%[ :]%[^\r\n]", &h, &m, &s, &c, buffer_text ) == 5 )
+        {
+            i_start = ( (mtime_t)h * 3600*1000 +
+                        (mtime_t)m * 60*1000 +
+                        (mtime_t)s * 1000 ) * 1000;
+            break;
+        }
+    }
+
+    /* replace | by \n */
+    for( i = 0; i < strlen( buffer_text ); i++ )
+    {
+        if( buffer_text[i] == '|' )
+        {
+            buffer_text[i] = '\n';
+        }
+    }
+    p_subtitle->i_start = i_start;
+
+    p_subtitle->i_stop  = 0;
+    p_subtitle->psz_text = strndup( buffer_text, MAX_LINE );
+    return( 0 );
+}
+
+static char *sub_SamiSearch( text_t *txt, char *psz_start, char *psz_str )
+{
+    if( psz_start )
+    {
+        if( strstr( psz_start, psz_str ) )
+        {
+            char *s = strstr( psz_start, psz_str );
+
+            s += strlen( psz_str );
+
+            return( s );
+        }
+    }
+    for( ;; )
+    {
+        char *p;
+        if( ( p = text_get_line( txt ) ) == NULL )
+        {
+            return NULL;
+        }
+        if( strstr( p, psz_str ) )
+        {
+            char *s = strstr( p, psz_str );
+
+            s += strlen( psz_str );
+
+            return(  s);
+        }
+    }
+}
+
+static int  sub_Sami( text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe )
+{
+    char *p;
+    int i_start;
+
+    int  i_text;
+    char buffer_text[10*MAX_LINE + 1];
+#define ADDC( c ) \
+    if( i_text < 10*MAX_LINE )      \
+    {                               \
+        buffer_text[i_text++] = c;  \
+        buffer_text[i_text] = '\0'; \
+    }
+
+    /* search "Start=" */
+    if( !( p = sub_SamiSearch( txt, NULL, "Start=" ) ) )
+    {
+        return VLC_EGENERIC;
+    }
+
+    /* get start value */
+    i_start = strtol( p, &p, 0 );
+
+    /* search <P */
+    if( !( p = sub_SamiSearch( txt, p, "<P" ) ) )
+    {
+        return VLC_EGENERIC;
+    }
+    /* search > */
+    if( !( p = sub_SamiSearch( txt, p, ">" ) ) )
+    {
+        return VLC_EGENERIC;
+    }
+
+    i_text = 0;
+    buffer_text[0] = '\0';
+    /* now get all txt until  a "Start=" line */
+    for( ;; )
+    {
+        if( *p )
+        {
+            if( *p == '<' )
+            {
+                if( !strncmp( p, "<br", 3 ) || !strncmp( p, "<BR", 3 ) )
+                {
+                    ADDC( '\n' );
+                }
+                else if( strstr( p, "Start=" ) )
+                {
+                    text_previous_line( txt );
+                    break;
+                }
+                p = sub_SamiSearch( txt, p, ">" );
+            }
+            else if( !strncmp( p, "&nbsp;", 6 ) )
+            {
+                ADDC( ' ' );
+                p += 6;
+            }
+            else if( *p == '\t' )
+            {
+                ADDC( ' ' );
+                p++;
+            }
+            else
+            {
+                ADDC( *p );
+                p++;
+            }
+        }
+        else
+        {
+            p = text_get_line( txt );
+        }
+
+        if( p == NULL )
+        {
+            break;
+        }
+    }
+
+    p_subtitle->i_start = i_start * 1000;
+    p_subtitle->i_stop  = 0;
+    p_subtitle->psz_text = strndup( buffer_text, 10*MAX_LINE );
+
+    return( VLC_SUCCESS );
+#undef ADDC
 }
 
