@@ -58,7 +58,6 @@ KInterface::KInterface( intf_thread_t *p_intf, QWidget *parent,
     connect( fTimer, SIGNAL( timeout() ), this, SLOT( slotManage() ) );
 
     resize( 400, 30 );
-    msg_Dbg(p_intf, KStdAction::stdName(KStdAction::Preferences));
     ///////////////////////////////////////////////////////////////////
     // call inits to invoke all other construction parts
     // XXX could we move this up ?
@@ -290,6 +289,13 @@ void KInterface::slotManage()
     {
         p_intf->p_sys->p_input = (input_thread_t *)
                 vlc_object_find( p_intf, VLC_OBJECT_INPUT, FIND_ANYWHERE );
+        if ( p_intf->p_sys->p_input )
+        {
+            languages->setEnabled( true );
+            subtitles->setEnabled( true );
+            info->setEnabled( true );
+        }
+            
     }
     else if( p_intf->p_sys->p_input->b_dead )
     {
