@@ -2,7 +2,7 @@
  * intf.m: MacOS X interface plugin
  *****************************************************************************
  * Copyright (C) 2002-2003 VideoLAN
- * $Id: intf.m,v 1.58 2003/02/16 23:32:06 hartman Exp $
+ * $Id: intf.m,v 1.69 2003/04/06 23:21:13 massiot Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -181,6 +181,11 @@ static void Run( intf_thread_t *p_intf )
     return psz_string;
 }
 
+- (NSStringEncoding)getEncoding
+{
+    return i_encoding;
+}
+
 - (void)setIntf:(intf_thread_t *)_p_intf
 {
     p_intf = _p_intf;
@@ -288,7 +293,6 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
     [o_msgs_panel setDelegate: self];
     [o_msgs_panel setTitle: _NS("Messages")];
     [o_msgs_panel setExcludedFromWindowsMenu: TRUE];
-    [o_msgs_btn_ok setTitle: _NS("Close")];
     [o_msgs_btn_crashlog setTitle: _NS("Open CrashLog")];
 
     /* main menu */
@@ -299,7 +303,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
     [o_mi_show_all setTitle: _NS("Show All")];
     [o_mi_quit setTitle: _NS("Quit VLC")];
 
-    [o_mu_file setTitle: _NS("File")];
+    [o_mu_file setTitle: _ANS("1:File")];
     [o_mi_open_generic setTitle: _NS("Open...")];
     [o_mi_open_file setTitle: _NS("Open File...")];
     [o_mi_open_disc setTitle: _NS("Open Disc...")];
@@ -344,6 +348,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
     [o_mi_fullscreen setTitle: _NS("Fullscreen")];
     [o_mi_screen setTitle: _NS("Screen")];
     [o_mi_deinterlace setTitle: _NS("Deinterlace")];
+    [o_mu_deinterlace setTitle: _NS("Deinterlace")];
 
     [o_mu_window setTitle: _NS("Window")];
     [o_mi_minimize setTitle: _NS("Minimize Window")];
@@ -397,10 +402,12 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
     [NSTimer scheduledTimerWithTimeInterval: 0.5
         target: self selector: @selector(manageIntf:)
-        userInfo: nil repeats: TRUE];
+        userInfo: nil repeats: FALSE];
 
     [NSThread detachNewThreadSelector: @selector(manage)
         toTarget: self withObject: nil];
+
+    vlc_thread_set_priority( p_intf, VLC_THREAD_PRIORITY_LOW );
 }
 
 - (BOOL)application:(NSApplication *)o_app openFile:(NSString *)o_filename
@@ -411,6 +418,15 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
     return( TRUE );
 }
 
+- (id)getControls
+{
+    if ( o_controls )
+    {
+        return o_controls;
+    }
+    return nil;
+}
+
 - (void)manage
 {
     NSDate * o_sleep_date;
@@ -430,15 +446,6 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
         if( p_playlist != NULL )
         {
-            vlc_value_t val;
-
-            if( var_Get( (vlc_object_t *)p_playlist, "intf-change", &val )
-                >= 0 && val.b_bool )
-            {
-                p_intf->p_sys->b_playlist_update = 1;
-                p_intf->p_sys->b_intf_update = VLC_TRUE;
-            }
-            
             vlc_mutex_lock( &p_playlist->object_lock );
             
             [self manage: p_playlist];
@@ -460,8 +467,17 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
 - (void)manage:(playlist_t *)p_playlist
 {
+    vlc_value_t val;
+
     intf_thread_t * p_intf = [NSApp getIntf];
 
+    if( var_Get( (vlc_object_t *)p_playlist, "intf-change", &val ) >= 0 &&
+        val.b_bool )
+    {
+        p_intf->p_sys->b_playlist_update = VLC_TRUE;
+        p_intf->p_sys->b_intf_update = VLC_TRUE;
+    }
+
 #define p_input p_playlist->p_input
 
     if( p_input )
@@ -503,8 +519,10 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
                     p_intf->p_sys->b_aout_update = 1;
                     b_need_menus = VLC_TRUE;
                 }
+
                 vlc_object_release( (vlc_object_t *)p_aout );
             }
+
             aout_VolumeGet( p_intf, &i_volume );
             p_intf->p_sys->b_mute = ( i_volume == 0 );
 
@@ -573,15 +591,6 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
             vout_Destroy( p_vout );
             vlc_mutex_lock( &p_playlist->object_lock );
         }
-
-        aout_instance_t * p_aout = vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                                            FIND_ANYWHERE );
-        if( p_aout != NULL )
-        {
-            vlc_object_detach( (vlc_object_t *)p_aout );
-            vlc_object_release( (vlc_object_t *)p_aout );
-            aout_Delete( p_aout ); 
-        }
     }
 
     p_intf->p_sys->b_intf_update = VLC_TRUE;
@@ -605,51 +614,46 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
         return;
     }
 
-    vlc_mutex_lock( &p_playlist->object_lock );
-    
     if ( p_intf->p_sys->b_playlist_update )
     {
         vlc_value_t val;
-        val.b_bool = 0;
 
+        val.b_bool = 0;
         var_Set( (vlc_object_t *)p_playlist, "intf-change", val );
+
         [o_playlist playlistUpdated];
+
         p_intf->p_sys->b_playlist_update = VLC_FALSE;
     }
 
+    if( p_intf->p_sys->b_current_title_update )
+    {
+        id o_vout_wnd;
+
+        NSEnumerator * o_enum = [[NSApp windows] objectEnumerator];
+        
+        while( ( o_vout_wnd = [o_enum nextObject] ) )
+        {
+            if( [[o_vout_wnd className] isEqualToString: @"VLCWindow"] )
+            {
+                [o_vout_wnd updateTitle];
+            }
+        }
+
+        [o_playlist updateRowSelection];
+        [o_info updateInfo];
+
+        p_intf->p_sys->b_current_title_update = FALSE;
+    }
+
+    vlc_mutex_lock( &p_playlist->object_lock );
+
 #define p_input p_playlist->p_input
 
     if( p_input != NULL )
     {
         vlc_mutex_lock( &p_input->stream.stream_lock );
     }
-    
-    if( p_intf->p_sys->b_current_title_update )
-    {
-        id o_awindow = [NSApp keyWindow];
-        NSArray *o_windows = [NSApp windows];
-        NSEnumerator *o_enumerator = [o_windows objectEnumerator];
-        
-        while ((o_awindow = [o_enumerator nextObject]))
-        {
-            if( [[o_awindow className] isEqualToString: @"VLCWindow"] )
-            {
-                vlc_mutex_unlock( &p_playlist->object_lock );
-                [o_awindow updateTitle];
-                vlc_mutex_lock( &p_playlist->object_lock );
-            }
-        }
-        vlc_mutex_unlock( &p_playlist->object_lock );
-        [o_playlist updateState];
-        vlc_mutex_lock( &p_playlist->object_lock );
-        if( p_input != NULL )
-        {
-            vlc_mutex_unlock( &p_input->stream.stream_lock );
-            [o_info updateInfo];
-            vlc_mutex_lock( &p_input->stream.stream_lock );
-        }
-        p_intf->p_sys->b_current_title_update = FALSE;
-    }
 
     if( p_intf->p_sys->b_intf_update )
     {
@@ -720,7 +724,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
         {
             if( f_slider == f_slider_old )
             {
-                float f_updated = ( 100. * p_area->i_tell ) /
+                float f_updated = ( 10000. * p_area->i_tell ) /
                                            p_area->i_size;
 
                 if( f_slider != f_updated )
@@ -730,7 +734,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
             }
             else
             {
-                off_t i_seek = ( f_slider * p_area->i_size ) / 100;
+                off_t i_seek = ( f_slider * p_area->i_size ) / 10000;
 
                 /* release the lock to be able to seek */
                 vlc_mutex_unlock( &p_input->stream.stream_lock );
@@ -772,6 +776,10 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
     vlc_object_release( p_playlist );
 
     [self updateMessageArray];
+
+    [NSTimer scheduledTimerWithTimeInterval: 0.5
+        target: self selector: @selector(manageIntf:)
+        userInfo: nil repeats: FALSE];
 }
 
 - (void)updateMessageArray
@@ -806,7 +814,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
             [o_msg_lock lock];
 
-            if( [o_msg_arr count] + 2 > 200 )
+            if( [o_msg_arr count] + 2 > 400 )
             {
                 unsigned rid[] = { 0, 1 };
                 [o_msg_arr removeObjectsFromIndices: (unsigned *)&rid
@@ -1404,7 +1412,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
 #define p_area p_playlist->p_input->stream.p_selected_area
         vlc_mutex_lock( &p_playlist->p_input->stream.stream_lock );
-        i_tell = f_updated / 100. * p_area->i_size;
+        i_tell = f_updated / 10000. * p_area->i_size;
         input_OffsetToTime( p_playlist->p_input, psz_time, i_tell );
         vlc_mutex_unlock( &p_playlist->p_input->stream.stream_lock );
 #undef p_area
