@@ -2,7 +2,7 @@
  * playlist.m: MacOS X interface plugin
  *****************************************************************************
  * Copyright (C) 2002-2003 VideoLAN
- * $Id: playlist.m,v 1.14 2003/03/06 15:24:12 hartman Exp $
+ * $Id: playlist.m,v 1.17 2003/03/17 23:13:06 hartman Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Derk-Jan Hartman <thedj@users.sourceforge.net>
@@ -37,6 +37,15 @@
  *****************************************************************************/
 @implementation VLCPlaylistView
 
+- (void)dealloc
+{
+    if( o_striped_row_color != nil )
+    {
+        [o_striped_row_color release];
+    }
+    [super dealloc];
+}
+
 - (NSMenu *)menuForEvent:(NSEvent *)o_event
 {
     return( [[self delegate] menuForEvent: o_event] );
@@ -102,6 +111,36 @@
     }
 }
 
+- (void) highlightSelectionInClipRect:(NSRect)o_rect {
+    NSRect o_new_rect;
+    float f_height = [self rowHeight] + [self intercellSpacing].height;
+    float f_origin_y = NSMaxY( o_rect );
+    int i_row = o_rect.origin.y / f_height;
+    
+    if ( i_row % 2 == 0 )
+    {
+        i_row++;
+    }
+    
+    o_new_rect.size.width = o_rect.size.width;
+    o_new_rect.size.height = f_height;
+    o_new_rect.origin.x = o_rect.origin.x;
+    o_new_rect.origin.y = i_row * f_height;
+    
+    if ( o_striped_row_color == nil )
+    {
+        o_striped_row_color = [[[NSColor alternateSelectedControlColor]
+                                highlightWithLevel: 0.90] retain];
+    }
+    [o_striped_row_color set];
+    
+    while ( o_new_rect.origin.y < f_origin_y ) {
+        NSRectFill( o_new_rect );
+        o_new_rect.origin.y += f_height * 2.0;
+    }
+    [super highlightSelectionInClipRect:o_rect];
+}
+
 @end
 
 /*****************************************************************************
@@ -109,6 +148,16 @@
  *****************************************************************************/
 @implementation VLCPlaylist
 
+- (id)init
+{
+    self = [super init];
+    if ( self !=nil )
+    {
+        i_moveRow = -1;
+    }
+    return self;
+}
+
 - (void)awakeFromNib
 {
     [o_table_view setTarget: self];
@@ -135,56 +184,6 @@
     return( NO );
 }
 
-- (NSDragOperation)tableView:(NSTableView*)o_tv 
-                   validateDrop:(id <NSDraggingInfo>)info 
-                   proposedRow:(int)i_row 
-                   proposedDropOperation:(NSTableViewDropOperation)operation
-{
-    return( NSDragOperationPrivate );
-}
-
-- (BOOL)tableView:(NSTableView*)o_tv 
-                  acceptDrop:(id <NSDraggingInfo>)info 
-                  row:(int)i_row 
-                  dropOperation:(NSTableViewDropOperation)operation
-{
-    NSArray * o_values;
-    NSPasteboard * o_pasteboard;
-
-    o_pasteboard = [info draggingPasteboard];
-
-    if( [[o_pasteboard types] containsObject: NSFilenamesPboardType] )
-    {
-        o_values = [o_pasteboard propertyListForType: NSFilenamesPboardType];
-
-        [self appendArray: o_values atPos: i_row enqueue:YES];
-
-        return( YES );
-    }
-
-    return( NO ); 
-}
-
-- (void)tableView:(NSTableView *)o_tv willDisplayCell:(id)o_cell
-                  forTableColumn:(NSTableColumn *)o_tc row:(int)i_row
-{
-    NSColor * o_color;
-
-    [o_cell setDrawsBackground: YES];
-
-    if( i_row % 2 )
-    {
-        o_color = [[NSColor alternateSelectedControlColor]
-                            highlightWithLevel: 0.90];
-    }
-    else
-    {
-        o_color = [o_tv backgroundColor]; 
-    }
-
-    [o_cell setBackgroundColor: o_color];
-}
-
 - (NSMenu *)menuForEvent:(NSEvent *)o_event
 {
     NSPoint pt;
@@ -372,5 +371,91 @@
     return( o_value );
 }
 
+- (BOOL)tableView:(NSTableView *)o_tv
+                    writeRows:(NSArray*)o_rows
+                    toPasteboard:(NSPasteboard*)o_pasteboard 
+{
+    int i_rows = [o_rows count];
+    NSArray *o_filenames = [NSArray array];
+    
+    [o_pasteboard declareTypes:[NSArray arrayWithObject:NSFilenamesPboardType] owner:self];
+    [o_pasteboard setPropertyList:o_filenames forType:NSFilenamesPboardType];
+    if ( i_rows == 1 )
+    {
+        i_moveRow = [[o_rows objectAtIndex:0]intValue];
+        return YES;
+    }
+    return NO;
+}
+
+- (NSDragOperation)tableView:(NSTableView*)o_tv
+                    validateDrop:(id <NSDraggingInfo>)o_info
+                    proposedRow:(int)i_row
+                    proposedDropOperation:(NSTableViewDropOperation)o_operation 
+{
+    if ( o_operation == NSTableViewDropAbove )
+    {
+        if ( i_moveRow >= 0 )
+        {
+            if ( i_row != i_moveRow )
+            {
+                return NSDragOperationMove;
+            }
+            /* what if in the previous run, the row wasn't actually moved? 
+               then we can't drop new files on this location */
+            return NSDragOperationNone;
+        }
+        return NSDragOperationGeneric;
+    }
+    return NSDragOperationNone;
+}
+
+- (BOOL)tableView:(NSTableView*)o_tv
+                    acceptDrop:(id <NSDraggingInfo>)o_info
+                    row:(int)i_proposed_row
+                    dropOperation:(NSTableViewDropOperation)o_operation 
+{
+    if (  i_moveRow >= 0 )
+    {
+        if (i_moveRow != -1 && i_proposed_row != -1)
+        {
+            intf_thread_t * p_intf = [NSApp getIntf];
+            playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                                            FIND_ANYWHERE );
+        
+            if( p_playlist == NULL )
+            {
+                i_moveRow = -1;
+                return NO;
+            }
+    
+            playlist_Move( p_playlist, i_moveRow, i_proposed_row ); 
+        
+            vlc_object_release( p_playlist );
+        }
+        [self playlistUpdated];
+        i_moveRow = -1;
+        return YES;
+    }
+    else
+    {
+        NSArray * o_values;
+        NSPasteboard * o_pasteboard;
+        
+        o_pasteboard = [o_info draggingPasteboard];
+        
+        if( [[o_pasteboard types] containsObject: NSFilenamesPboardType] )
+        {
+            o_values = [o_pasteboard propertyListForType: NSFilenamesPboardType];
+        
+            [self appendArray: o_values atPos: i_proposed_row enqueue:YES];
+        
+            return( YES );
+        }
+        
+        return( NO );
+    }
+}
+
 @end
 
