@@ -1,8 +1,8 @@
 /*****************************************************************************
  * prefs.m: MacOS X plugin for vlc
  *****************************************************************************
- * Copyright (C) 2002 VideoLAN
- * $Id: prefs.m,v 1.12 2003/02/09 19:28:43 massiot Exp $
+ * Copyright (C) 2002-2003 VideoLAN
+ * $Id: prefs.m,v 1.17 2003/03/06 10:15:37 hartman Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *
@@ -236,8 +236,11 @@
         [o_text_field setBordered: NO]; \
         [o_text_field setEditable: NO]; \
         [o_text_field setSelectable: NO]; \
-        [o_text_field setStringValue: \
-            [NSApp localizedString: label]]; \
+        if ( label ) \
+        { \
+            [o_text_field setStringValue: \
+                [NSApp localizedString: label]]; \
+        } \
         [o_text_field sizeToFit]; \
         [o_view addSubview: [o_text_field autorelease]]; \
     }
@@ -245,7 +248,7 @@
 #define INPUT_FIELD( ctype, cname, label, w, msg, param, tip ) \
     { \
         char * psz_duptip = NULL; \
-        if ( p_item->psz_longtext != NULL ) \
+        if ( p_item->psz_longtext != NULL && [NSApp getEncoding] == NSISOLatin1StringEncoding ) \
             psz_duptip = strdup(p_item->psz_longtext); \
         s_rc.size.height = 25; \
         s_rc.size.width = w; \
@@ -280,7 +283,10 @@
 
     if( p_item ) do
     {
-
+        if( p_item->b_advanced && !config_GetInt( p_intf, "advanced" ) )
+        {
+            continue;
+        }
         switch( p_item->i_type )
         {
 
@@ -328,7 +334,7 @@
             NSButton *o_btn_select;
             NSButton *o_btn_configure;
             char * psz_duptip = NULL;
-            if ( p_item->psz_longtext != NULL )
+            if ( p_item->psz_longtext != NULL && [NSApp getEncoding] == NSISOLatin1StringEncoding )
                 psz_duptip = strdup(p_item->psz_longtext);
 
 #define MODULE_BUTTON( button, title, sel ) \
@@ -448,7 +454,7 @@
                 int i;
                 VLCComboBox *o_combo_box;
                 char * psz_duptip = NULL;
-                if ( p_item->psz_longtext != NULL )
+                if ( p_item->psz_longtext != NULL && [NSApp getEncoding] == NSISOLatin1StringEncoding )
                     psz_duptip = strdup(p_item->psz_longtext);
 
                 s_rc.size.height = 27;
@@ -509,7 +515,7 @@
         {
             VLCButton *o_btn_bool;
             char * psz_duptip = NULL;
-            if ( p_item->psz_longtext != NULL )
+            if ( p_item->psz_longtext != NULL && [NSApp getEncoding] == NSISOLatin1StringEncoding )
                 psz_duptip = strdup(p_item->psz_longtext);
 
             s_rc.size.height = 27;
@@ -573,15 +579,15 @@
     s_rc.origin.y = s_panel_rc.origin.y + 14;
     s_rc.size.height = 25; s_rc.size.width = 105;
     s_rc.origin.x = s_panel_rc.size.width - s_rc.size.width - 14;
-    DEF_PANEL_BUTTON( 0, _NS("OK"), clickedCancelOK: );
+    DEF_PANEL_BUTTON( 0, _NS("OK"), clickedApplyCancelOK: );
     [o_panel setDefaultButtonCell: [o_button cell]];
 
     s_rc.origin.x -= s_rc.size.width;
-    DEF_PANEL_BUTTON( 1, _NS("Cancel"), clickedCancelOK: );
+    DEF_PANEL_BUTTON( 1, _NS("Cancel"), clickedApplyCancelOK: );
     [o_button setKeyEquivalent: @"\E"];
 
     s_rc.origin.x -= s_rc.size.width;
-    DEF_PANEL_BUTTON( 2, _NS("Apply"), clickedApply: );
+    DEF_PANEL_BUTTON( 2, _NS("Apply"), clickedApplyCancelOK: );
     [o_button setEnabled: NO];
 
 #undef DEF_PANEL_BUTTON
@@ -721,92 +727,102 @@
     }
 }
 
-- (void)clickedApply:(id)sender
+- (void)clickedApplyCancelOK:(id)sender
 {
     id o_vlc_control;
     NSEnumerator *o_enum;
-
-    NSView *o_config_view = [sender superview];
-    NSWindow *o_config_panel = [o_config_view window];
-    NSButton *o_btn_apply = [o_config_view viewWithTag: 2];
-    NSString *o_module_name = [[o_config_panel toolbar] identifier];
-    NSMutableArray *o_prefs = [o_save_prefs objectForKey: o_module_name];
-
-    o_enum = [o_prefs objectEnumerator];
-    while( ( o_vlc_control = [o_enum nextObject] ) )
-    {
-        int i_type = [o_vlc_control configType];
-        NSString *o_name = [o_vlc_control configName];
-        char *psz_name = (char *)[o_name lossyCString];
-
-        switch( i_type )
-        {
-
-        case CONFIG_ITEM_MODULE:
-        case CONFIG_ITEM_STRING:
-        case CONFIG_ITEM_FILE:
-            {
-                char *psz_value;
-                NSString *o_value;
-
-                o_value = [o_vlc_control stringValue];
-                psz_value = (char *)[o_value lossyCString];
-
-                config_PutPsz( p_intf, psz_name,
-                               *psz_value ? psz_value : NULL );
-            }
-            break;
-
-        case CONFIG_ITEM_INTEGER:
-        case CONFIG_ITEM_BOOL:
-            {
-                int i_value = [o_vlc_control intValue];
-
-                config_PutInt( p_intf, psz_name, i_value );
-            }
-            break;
-
-        case CONFIG_ITEM_FLOAT:
-            {
-                float f_value = [o_vlc_control floatValue];
-
-                config_PutFloat( p_intf, psz_name, f_value );
-            }
-            break;
-
-        }
-    }
-
-    [o_btn_apply setEnabled: NO];
-    [o_prefs removeAllObjects];
-
-    config_SaveConfigFile( p_intf, NULL );
-}
-
-- (void)clickedCancelOK:(id)sender
-{
+    BOOL b_adv_change = FALSE;
+    
     NSWindow *o_pref_panel = [[sender superview] window];
     NSString *o_module_name = [[o_pref_panel toolbar] identifier];
 
-    if( [[sender title] isEqualToString: _NS("OK")] )
+    if ( ![[sender title] isEqualToString: _NS("Cancel")] )
     {
-        [self clickedApply: sender];
-    }
+        NSView *o_config_view = [sender superview];
+        NSWindow *o_config_panel = [o_config_view window];
+        NSButton *o_btn_apply = [o_config_view viewWithTag: 2];
+        NSString *o_module_name = [[o_config_panel toolbar] identifier];
+        NSMutableArray *o_prefs = [o_save_prefs objectForKey: o_module_name];
+    
+        o_enum = [o_prefs objectEnumerator];
+        while( ( o_vlc_control = [o_enum nextObject] ) )
+        {
+            int i_type = [o_vlc_control configType];
+            NSString *o_name = [o_vlc_control configName];
+            char *psz_name = (char *)[o_name lossyCString];
+    
+            switch( i_type )
+            {
+    
+            case CONFIG_ITEM_MODULE:
+            case CONFIG_ITEM_STRING:
+            case CONFIG_ITEM_FILE:
+                {
+                    char *psz_value;
+                    NSString *o_value;
+    
+                    o_value = [o_vlc_control stringValue];
+                    psz_value = (char *)[o_value lossyCString];
+    
+                    config_PutPsz( p_intf, psz_name,
+                                   *psz_value ? psz_value : NULL );
+                }
+                break;
+    
+            case CONFIG_ITEM_INTEGER:
+            case CONFIG_ITEM_BOOL:
+                {
+                    int i_value = [o_vlc_control intValue];
 
-    [o_pref_panel close];
+                    if( !strcmp( psz_name, "advanced" ) && 
+                        ( config_GetInt( p_intf, "advanced" ) != i_value ) )
+                    {
+                        b_adv_change = TRUE;
+                    }
 
-    if( [self respondsToSelector: @selector(performSelectorOnMainThread:
-                                            withObject:waitUntilDone:)] )
-    {
-        [self performSelectorOnMainThread: @selector(destroyPrefPanel:)
-                                           withObject: o_module_name
-                                           waitUntilDone: NO];
+                    config_PutInt( p_intf, psz_name, i_value );
+                }
+                break;
+    
+            case CONFIG_ITEM_FLOAT:
+                {
+                    float f_value = [o_vlc_control floatValue];
+    
+                    config_PutFloat( p_intf, psz_name, f_value );
+                }
+                break;
+    
+            }
+        }
+    
+        [o_btn_apply setEnabled: NO];
+        [o_prefs removeAllObjects];
+    
+        config_SaveConfigFile( p_intf, NULL );
     }
-    else
+    
+    if( ![[sender title] isEqualToString: _NS("Apply")] || b_adv_change )
     {
-        [NSTimer scheduledTimerWithTimeInterval: 0.1
-                 target: self selector: @selector(destroyPrefPanel:)
-                 userInfo: o_module_name repeats: NO];
+        [o_pref_panel close];
+
+        if( [self respondsToSelector: @selector(performSelectorOnMainThread:
+                                                withObject:waitUntilDone:)] )
+        {
+            [self performSelectorOnMainThread: @selector(destroyPrefPanel:)
+                                            withObject: o_module_name
+                                            waitUntilDone: YES];
+        }
+        else
+        {
+            [NSTimer scheduledTimerWithTimeInterval: 0.1
+                    target: self selector: @selector(destroyPrefPanel:)
+                    userInfo: o_module_name repeats: NO];
+        }
+
+        if( [[sender title] isEqualToString: _NS("Apply")] && b_adv_change )
+        {
+            [self createPrefPanel: o_module_name];
+        }
     }
 }
 
