@@ -2,7 +2,7 @@
  * interface.cpp : wxWindows plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000-2001 VideoLAN
- * $Id: interface.cpp,v 1.15 2003/01/26 13:37:09 gbazin Exp $
+ * $Id: interface.cpp,v 1.20 2003/04/07 09:25:54 gbazin Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -112,6 +112,7 @@ BEGIN_EVENT_TABLE(Interface, wxFrame)
     EVT_MENU(Playlist_Event, Interface::OnPlaylist)
     EVT_MENU(Logs_Event, Interface::OnLogs)
     EVT_MENU(FileInfo_Event, Interface::OnFileInfo)
+    EVT_MENU(Prefs_Event, Interface::OnPreferences)
 
     /* Toolbar events */
     EVT_MENU(OpenFile_Event, Interface::OnOpenFile)
@@ -136,10 +137,12 @@ Interface::Interface( intf_thread_t *_p_intf ):
 {
     /* Initializations */
     p_intf = _p_intf;
-    i_playing_status = PAUSE_S;
+    p_prefs_dialog = NULL;
+    i_old_playing_status = PAUSE_S;
 
     /* Give our interface a nice little icon */
-    SetIcon( *new wxIcon( vlc_xpm ) );
+    p_intf->p_sys->p_icon = new wxIcon( vlc_xpm );
+    SetIcon( *p_intf->p_sys->p_icon );
 
     /* Create a sizer for the main frame */
     frame_sizer = new wxBoxSizer( wxHORIZONTAL );
@@ -153,6 +156,8 @@ Interface::Interface( intf_thread_t *_p_intf ):
 
     /* Creation of the slider sub-window */
     CreateOurSlider();
+    frame_sizer->Add( slider_frame, 1, wxGROW, 0 );
+    frame_sizer->Hide( slider_frame );
 
     /* Creation of the status bar
      * Helptext for menu items and toolbar tools will automatically get
@@ -177,6 +182,7 @@ Interface::Interface( intf_thread_t *_p_intf ):
 
 Interface::~Interface()
 {
+    if( p_prefs_dialog ) p_prefs_dialog->Destroy();
 }
 
 /*****************************************************************************
@@ -313,7 +319,6 @@ void Interface::CreateOurSlider()
     /* Create a new frame containing the slider */
     slider_frame = new wxPanel( this, -1, wxDefaultPosition, wxDefaultSize );
     slider_frame->SetAutoLayout( TRUE );
-    slider_frame->Hide();
 
     /* Create static box to surround the slider */
     slider_box = new wxStaticBox( slider_frame, -1, "" );
@@ -330,6 +335,9 @@ void Interface::CreateOurSlider()
     slider_sizer->Add( slider, 1, wxGROW | wxALL, 5 );
     slider_sizer->Layout();
     slider_sizer->SetSizeHints(slider_frame);
+
+    /* Hide the slider by default */
+    slider_frame->Hide();
 }
 
 void Interface::Open( int i_access_method )
@@ -348,13 +356,10 @@ void Interface::Open( int i_access_method )
             return;
         }
 
-        msg_Err( p_intf, "%s", (char *)dialog.mrl.c_str() );
-
         playlist_Add( p_playlist, (char *)dialog.mrl.c_str(),
                       PLAYLIST_APPEND | PLAYLIST_GO, PLAYLIST_END );
 
-        i_playing_status = PLAYING_S;
-        TogglePlayButton();
+        TogglePlayButton( PLAYING_S );
 
         /* Rebuild the playlist */
         p_intf->p_sys->p_playlist_window->Rebuild();
@@ -383,7 +388,7 @@ void Interface::OnAbout( wxCommandEvent& WXUNUSED(event) )
           "\nIt can play MPEG and MPEG2 files from a file or from a "
           "network source.")) );
 
-    wxMessageBox( msg, wxString(_("About ")) + VOUT_TITLE,
+    wxMessageBox( msg, wxString::Format(_("About %s"), VOUT_TITLE),
                   wxOK | wxICON_INFORMATION, this );
 }
 
@@ -409,11 +414,21 @@ void Interface::OnLogs( wxCommandEvent& WXUNUSED(event) )
 
 void Interface::OnFileInfo( wxCommandEvent& WXUNUSED(event) )
 {
-    /* Show/hide the fileinfo window */
-    wxFrame *p_fileinfo_window = new FileInfo( p_intf, this );
-    if( p_fileinfo_window )
+    /* Open a fileinfo window */
+    new FileInfo( p_intf, this );
+}
+
+void Interface::OnPreferences( wxCommandEvent& WXUNUSED(event) )
+{
+    /* Show/hide the open dialog */
+    if( p_prefs_dialog == NULL )
     {
-        p_fileinfo_window->Show( true );//! p_messages_window->IsShown() );
+        p_prefs_dialog = new PrefsDialog( p_intf, this );
+    }
+
+    if( p_prefs_dialog )
+    {
+        p_prefs_dialog->Show( true );
     }
 }
 
@@ -445,11 +460,11 @@ void Interface::OnPlayStream( wxCommandEvent& WXUNUSED(event) )
                                        FIND_ANYWHERE );
     if( p_playlist == NULL )
     {
+        /* If the playlist is empty, open a file requester instead */
         OnOpenFile( dummy );
         return;
     }
 
-    /* If the playlist is empty, open a file requester instead */
     vlc_mutex_lock( &p_playlist->object_lock );
     if( p_playlist->i_size )
     {
@@ -462,8 +477,7 @@ void Interface::OnPlayStream( wxCommandEvent& WXUNUSED(event) )
         {
             /* No stream was playing, start one */
             playlist_Play( p_playlist );
-            i_playing_status = PLAYING_S;
-            TogglePlayButton();
+            TogglePlayButton( PLAYING_S );
             vlc_object_release( p_playlist );
             return;
         }
@@ -472,8 +486,7 @@ void Interface::OnPlayStream( wxCommandEvent& WXUNUSED(event) )
         {
             /* A stream is being played, pause it */
             input_SetStatus( p_input, INPUT_STATUS_PAUSE );
-            i_playing_status = PAUSE_S;
-            TogglePlayButton();
+            TogglePlayButton( PAUSE_S );
             vlc_object_release( p_playlist );
             vlc_object_release( p_input );
             return;
@@ -481,8 +494,7 @@ void Interface::OnPlayStream( wxCommandEvent& WXUNUSED(event) )
 
         /* Stream is paused, resume it */
         playlist_Play( p_playlist );
-        i_playing_status = PLAYING_S;
-        TogglePlayButton();
+        TogglePlayButton( PLAYING_S );
         vlc_object_release( p_input );
         vlc_object_release( p_playlist );
     }
@@ -505,8 +517,7 @@ void Interface::OnStopStream( wxCommandEvent& WXUNUSED(event) )
     }
 
     playlist_Stop( p_playlist );
-    i_playing_status = PAUSE_S;
-    TogglePlayButton();
+    TogglePlayButton( PAUSE_S );
     vlc_object_release( p_playlist );
 }
 
@@ -582,8 +593,11 @@ void Interface::OnNextStream( wxCommandEvent& WXUNUSED(event) )
     vlc_object_release( p_playlist );
 }
 
-void Interface::TogglePlayButton( )
+void Interface::TogglePlayButton( int i_playing_status )
 {
+    if( i_playing_status == i_old_playing_status )
+        return;
+
     GetToolBar()->DeleteTool( PlayStream_Event );
 
     if( i_playing_status == PLAYING_S )
@@ -598,6 +612,8 @@ void Interface::TogglePlayButton( )
     }
 
     GetToolBar()->Realize();
+
+    i_old_playing_status = i_playing_status;
 }
 
 #if !defined(__WXX11__)
