@@ -2,7 +2,7 @@
  * motion_blur.c : motion blur filter for vlc
  *****************************************************************************
  * Copyright (C) 2000, 2001, 2002, 2003 VideoLAN
- * $Id: motionblur.c,v 1.8 2003/02/20 01:52:46 sigmunau Exp $
+ * $Id: motionblur.c,v 1.10 2003/03/30 18:14:38 gbazin Exp $
  *
  * Authors: Sigmund Augdal <sigmunau@idi.ntnu.no>
  *
@@ -57,7 +57,7 @@ static int  SendEvents( vlc_object_t *, char const *,
 vlc_module_begin();
     add_category_hint( N_("Miscellaneous"), NULL, VLC_FALSE );
     add_integer( "blur-factor", 80, NULL, MODE_TEXT, MODE_LONGTEXT, VLC_FALSE );
-    set_description( _("Motion blur filter") );
+    set_description( _("motion blur filter") );
     set_capability( "video filter", 0 );
     set_callbacks( Create, Destroy );
 vlc_module_end();
@@ -195,6 +195,7 @@ static void Destroy( vlc_object_t *p_this )
     vout_thread_t *p_vout = (vout_thread_t *)p_this;
 
     DEL_CALLBACKS( p_vout->p_sys->p_vout, SendEvents );
+    vlc_object_detach( p_vout->p_sys->p_vout );
     vout_Destroy( p_vout->p_sys->p_vout );
 
     free( p_vout->p_sys );
