@@ -4,7 +4,7 @@
  * decoders.
  *****************************************************************************
  * Copyright (C) 1998-2002 VideoLAN
- * $Id: input.c,v 1.221 2002/12/31 01:54:36 massiot Exp $
+ * $Id: input.c,v 1.228 2003/03/30 01:13:37 massiot Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -45,6 +45,7 @@
 #include "input_ext-plugins.h"
 
 #include "stream_output.h"
+#include <vlc/vout.h>
 
 #include "interface.h"
 
@@ -126,7 +127,7 @@ input_thread_t *__input_CreateThread( vlc_object_t *p_parent,
     p_input->stream.p_newly_selected_es = NULL;
 
     /* By default there is one area in a stream */
-    input_AddArea( p_input );
+    input_AddArea( p_input, 0, 1 );
     p_input->stream.p_selected_area = p_input->stream.pp_areas[0];
 
     /* Initialize stream control properties. */
@@ -148,8 +149,8 @@ input_thread_t *__input_CreateThread( vlc_object_t *p_parent,
 
     msg_Info( p_input, "playlist item `%s'", p_input->psz_source );
 
-    p_info = input_InfoCategory( p_input, "General" );
-    input_AddInfo( p_info, "playlist item", p_input->psz_source );
+    p_info = input_InfoCategory( p_input, _("General") );
+    input_AddInfo( p_info, _("Playlist Item"), p_input->psz_source );
     vlc_object_attach( p_input, p_parent );
 
     /* Create thread and wait for its readiness. */
@@ -341,7 +342,7 @@ static int RunThread( input_thread_t *p_input )
         /* Read and demultiplex some data. */
         i_count = p_input->pf_demux( p_input );
 
-        if( i_count == 0 && p_input->stream.b_connected )
+        if( i_count == 0 )
         {
             /* End of file - we do not set b_die because only the
              * playlist is allowed to do so. */
@@ -370,7 +371,7 @@ static int RunThread( input_thread_t *p_input )
 static int InitThread( input_thread_t * p_input )
 {
     /* Parse source string. Syntax : [[<access>][/<demux>]:][<source>] */
-    char * psz_parser = p_input->psz_source;
+    char * psz_parser = p_input->psz_dupsource = strdup(p_input->psz_source);
 
     /* Skip the plug-in names */
     while( *psz_parser && *psz_parser != ':' )
@@ -378,10 +379,10 @@ static int InitThread( input_thread_t * p_input )
         psz_parser++;
     }
 #if defined( WIN32 ) || defined( UNDER_CE )
-    if( psz_parser - p_input->psz_source == 1 )
+    if( psz_parser - p_input->psz_dupsource == 1 )
     {
         msg_Warn( p_input, "drive letter %c: found in source string",
-                           p_input->psz_source[0] ) ;
+                           p_input->psz_dupsource[0] ) ;
         psz_parser = "";
     }
 #endif
@@ -390,6 +391,8 @@ static int InitThread( input_thread_t * p_input )
     {
         p_input->psz_access = p_input->psz_demux = "";
         p_input->psz_name = p_input->psz_source;
+        free( p_input->psz_dupsource );
+        p_input->psz_dupsource = NULL;
     }
     else
     {
@@ -404,7 +407,7 @@ static int InitThread( input_thread_t * p_input )
         p_input->psz_name = psz_parser ;
 
         /* Come back to parse the access and demux plug-ins */
-        psz_parser = p_input->psz_source;
+        psz_parser = p_input->psz_dupsource;
 
         if( !*psz_parser )
         {
@@ -455,6 +458,20 @@ static int InitThread( input_thread_t * p_input )
     p_input->p_access = module_Need( p_input, "access",
                                      p_input->psz_access );
 
+    if ( p_input->p_access == NULL
+          && (*p_input->psz_demux || *p_input->psz_access) )
+    {
+        /* Maybe we got something like :
+         * /Volumes/toto:titi/gabu.mpg */
+        p_input->psz_access = p_input->psz_demux = "";
+        p_input->psz_name = p_input->psz_source;
+        free( p_input->psz_dupsource);
+        p_input->psz_dupsource = NULL;
+
+        p_input->p_access = module_Need( p_input, "access",
+                                         p_input->psz_access );
+    }
+
     if( p_input->p_access == NULL )
     {
         msg_Err( p_input, "no suitable access module for `%s/%s://%s'",
@@ -536,6 +553,8 @@ static void ErrorThread( input_thread_t *p_input )
  *****************************************************************************/
 static void EndThread( input_thread_t * p_input )
 {
+    vlc_object_t *p_object;
+
 #ifdef HAVE_SYS_TIMES_H
     /* Display statistics */
     struct tms  cpu_usage;
@@ -570,7 +589,18 @@ static void EndThread( input_thread_t * p_input )
 
     input_AccessEnd( p_input );
 
+    /* Close the video output that should have been re-attached
+     * to our object */
+    p_object = vlc_object_find( p_input, VLC_OBJECT_VOUT, FIND_CHILD );
+    if( p_object )
+    {
+        vlc_object_detach( p_object );
+        vlc_object_release( p_object );
+        vout_Destroy( (vout_thread_t *)p_object );
+    }
+
     free( p_input->psz_source );
+    if ( p_input->psz_dupsource != NULL ) free( p_input->psz_dupsource );
 
     /* Tell we're dead */
     p_input->b_dead = 1;
