@@ -2,7 +2,7 @@
  * input_ext-plugins.c: useful functions for access and demux plug-ins
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: input_ext-plugins.c,v 1.28 2003/02/06 23:55:29 massiot Exp $
+ * $Id: input_ext-plugins.c,v 1.31 2003/03/24 17:15:30 gbazin Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -494,6 +494,7 @@ ssize_t input_FillBuffer( input_thread_t * p_input )
     {
         vlc_mutex_unlock( &p_input->p_method_data->lock );
         msg_Err( p_input, "failed allocating a new buffer (decoder stuck?)" );
+        msleep( INPUT_IDLE_SLEEP );
         return -1;
     }
     p_buf->i_refcount = 1;
@@ -516,6 +517,12 @@ ssize_t input_FillBuffer( input_thread_t * p_input )
                               (byte_t *)p_buf + sizeof(data_buffer_t)
                                + i_remains,
                               p_input->i_bufsize );
+    if( i_ret < 0 && i_remains == 0 )
+    {
+        /* Our internal buffers are empty, we can signal the error */
+        return -1;
+    }
+
     if( i_ret < 0 ) i_ret = 0;
 
     p_input->p_data_buffer = p_buf;
@@ -537,7 +544,7 @@ ssize_t input_Peek( input_thread_t * p_input, byte_t ** pp_byte, size_t i_size )
         /* Go to the next buffer */
         ssize_t i_ret = input_FillBuffer( p_input );
 
-        if( i_ret == -1 )
+        if( i_ret < 0 )
         {
             return -1;
         }
@@ -563,7 +570,7 @@ ssize_t input_SplitBuffer( input_thread_t * p_input,
         /* Go to the next buffer */
         ssize_t i_ret = input_FillBuffer( p_input );
 
-        if( i_ret == -1 )
+        if( i_ret < 0 )
         {
             return -1;
         }
@@ -610,7 +617,7 @@ int input_AccessInit( input_thread_t * p_input )
 }
 
 /*****************************************************************************
- * input_AccessReinit: reinit structures after a random seek
+ * input_AccessReinit: reinit structures before a random seek
  *****************************************************************************/
 void input_AccessReinit( input_thread_t * p_input )
 {
