@@ -2,7 +2,7 @@
  * libvlc.c: main libvlc source
  *****************************************************************************
  * Copyright (C) 1998-2002 VideoLAN
- * $Id: libvlc.c,v 1.70 2003/03/04 23:36:57 massiot Exp $
+ * $Id: libvlc.c,v 1.77 2003/04/08 08:35:59 massiot Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -321,6 +321,29 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
         return VLC_EEXIT;
     }
 
+    /* Check for translation config option */
+#if defined( ENABLE_NLS ) \
+     && ( defined( HAVE_GETTEXT ) || defined( HAVE_INCLUDED_GETTEXT ) )
+
+    /* This ain't really nice to have to reload the config here but it seems
+     * the only way to do it. */
+    p_vlc->psz_homedir = config_GetHomeDir();
+    config_LoadConfigFile( p_vlc, "main" );
+    config_LoadCmdLine( p_vlc, &i_argc, ppsz_argv, VLC_TRUE );
+
+    if( !config_GetInt( p_vlc, "translation" ) )
+    {
+        /* Reset the default domain */
+        SetLanguage( "C" );
+
+        textdomain( "dummy" );
+
+        module_EndBank( p_vlc );
+        module_InitBank( &libvlc );
+        module_LoadMain( &libvlc );
+    }
+#endif
+
     /*
      * Load the builtins and plugins into the module_bank.
      * We have to do it before config_Load*() because this also gets the
@@ -507,7 +530,7 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
     psz_parser = psz_modules;
     while ( psz_parser && *psz_parser )
     {
-        char *psz_module;
+        char *psz_module, *psz_temp;
         psz_module = psz_parser;
         psz_parser = strchr( psz_module, ',' );
         if ( psz_parser )
@@ -515,7 +538,13 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
             *psz_parser = '\0';
             psz_parser++;
         }
-        VLC_AddIntf( 0, psz_module, VLC_FALSE );
+        psz_temp = (char *)malloc( strlen(psz_module) + sizeof(",none") );
+        if( psz_temp )
+        {
+            sprintf( psz_temp, "%s,none", psz_module );
+            VLC_AddIntf( 0, psz_temp, VLC_FALSE );
+            free( psz_temp );
+        }
     }
     if ( psz_modules )
     {
@@ -562,7 +591,7 @@ int VLC_AddIntf( int i_object, char const *psz_module, vlc_bool_t b_block )
 
     if( p_intf == NULL )
     {
-        msg_Err( p_vlc, "interface initialization failed" );
+        msg_Err( p_vlc, "interface \"%s\" initialization failed", psz_module );
         if( i_object ) vlc_object_release( p_vlc );
         return VLC_EGENERIC;
     }
@@ -996,10 +1025,22 @@ static void SetLanguage ( char const *psz_lang )
     }
 #   endif
 
+    if( psz_lang && !*psz_lang )
+    {
 #   if defined( HAVE_LC_MESSAGES )
-    setlocale( LC_MESSAGES, psz_lang );
+        setlocale( LC_MESSAGES, psz_lang );
 #   endif
-    setlocale( LC_CTYPE, psz_lang );
+        setlocale( LC_CTYPE, psz_lang );
+    }
+    else
+    {
+        setlocale( LC_ALL, psz_lang );
+#ifdef SYS_DARWIN
+        /* I need that under Darwin, please check it doesn't disturb
+         * other platforms. --Meuuh */
+        setenv( "LANG", psz_lang, 1 );
+#endif
+    }
 
     /* Specify where to find the locales for current domain */
 #if !defined( SYS_DARWIN ) && !defined( WIN32 )
@@ -1421,4 +1462,3 @@ static int ConsoleWidth( void )
 
     return i_width;
 }
-
