@@ -2,7 +2,7 @@
  * threads.c : threads implementation for the VideoLAN client
  *****************************************************************************
  * Copyright (C) 1999, 2000, 2001, 2002 VideoLAN
- * $Id: threads.c,v 1.36 2003/02/10 16:53:34 sam Exp $
+ * $Id: threads.c,v 1.40 2003/03/10 00:04:14 massiot Exp $
  *
  * Authors: Jean-Marc Dressler <polux@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -92,11 +92,10 @@ int __vlc_threads_init( vlc_object_t *p_this )
         i_status = VLC_THREADS_PENDING;
 
         /* We should be safe now. Do all the initialization stuff we want. */
-        vlc_object_create( p_libvlc, VLC_OBJECT_ROOT );
         p_libvlc->b_ready = VLC_FALSE;
 
 #if defined( PTH_INIT_IN_PTH_H )
-        i_ret = pth_init();
+        i_ret = ( pth_init() == FALSE );
 
 #elif defined( ST_INIT_IN_ST_H )
         i_ret = st_init();
@@ -130,6 +129,8 @@ int __vlc_threads_init( vlc_object_t *p_this )
 #elif defined( HAVE_KERNEL_SCHEDULER_H )
 #endif
 
+        vlc_object_create( p_libvlc, VLC_OBJECT_ROOT );
+
         if( i_ret )
         {
             i_status = VLC_THREADS_ERROR;
@@ -184,7 +185,7 @@ int __vlc_threads_end( vlc_object_t *p_this )
     i_initializations--;
     if( i_initializations == 0 )
     {
-        return pth_kill();
+        return ( pth_kill() == FALSE );
     }
 
 #elif defined( ST_INIT_IN_ST_H )
@@ -259,7 +260,7 @@ int __vlc_mutex_init( vlc_object_t *p_this, vlc_mutex_t *p_mutex )
     p_mutex->p_this = p_this;
 
 #if defined( PTH_INIT_IN_PTH_H )
-    return pth_mutex_init( &p_mutex->mutex );
+    return ( pth_mutex_init( &p_mutex->mutex ) == FALSE );
 
 #elif defined( ST_INIT_IN_ST_H )
     p_mutex->mutex = st_mutex_new();
@@ -401,7 +402,7 @@ int __vlc_cond_init( vlc_object_t *p_this, vlc_cond_t *p_condvar )
     p_condvar->p_this = p_this;
 
 #if defined( PTH_INIT_IN_PTH_H )
-    return pth_cond_init( &p_condvar->cond );
+    return ( pth_cond_init( &p_condvar->cond ) == FALSE );
 
 #elif defined( ST_INIT_IN_ST_H )
     p_condvar->cond = st_cond_new();
@@ -573,7 +574,7 @@ int __vlc_thread_create( vlc_object_t *p_this, char * psz_file, int i_line,
 
 #if defined( PTH_INIT_IN_PTH_H )
     p_this->thread_id = pth_spawn( PTH_ATTR_DEFAULT, func, p_data );
-    i_ret = 0;
+    i_ret = p_this->thread_id == NULL;
 
 #elif defined( ST_INIT_IN_ST_H )
     p_this->thread_id = st_thread_create( func, p_data, 1, 0 );
@@ -610,21 +611,31 @@ int __vlc_thread_create( vlc_object_t *p_this, char * psz_file, int i_line,
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
     i_ret = pthread_create( &p_this->thread_id, NULL, func, p_data );
 
+#ifdef SYS_DARWIN
     if ( i_priority )
     {
-        int i_error;
+        int i_error, i_policy;
         struct sched_param param;
         memset( &param, 0, sizeof(struct sched_param) );
-        param.sched_priority = i_priority;
-        if ( (i_error = pthread_setschedparam( p_this->thread_id,
-                                               SCHED_RR, &param )) )
+        if ( i_priority < 0 )
         {
-            msg_Warn( p_this, "couldn't go to real-time priority (%s:%d): %s",
+            param.sched_priority = (-1) * i_priority;
+            i_policy = SCHED_OTHER;
+        }
+        else
+        {
+            param.sched_priority = i_priority;
+            i_policy = SCHED_RR;
+        }
+        if ( (i_error = pthread_setschedparam( p_this->thread_id,
+                                               i_policy, &param )) )
+        {
+            msg_Warn( p_this, "couldn't set thread priority (%s:%d): %s",
                       psz_file, i_line, strerror(i_error) );
             i_priority = 0;
         }
-
     }
+#endif
 
 #elif defined( HAVE_CTHREADS_H )
     p_this->thread_id = cthread_fork( (cthread_fn_t)func, (any_t)p_data );
@@ -686,7 +697,8 @@ int __vlc_thread_create( vlc_object_t *p_this, char * psz_file, int i_line,
 int __vlc_thread_set_priority( vlc_object_t *p_this, char * psz_file,
                                int i_line, int i_priority )
 {
-#if defined( WIN32 ) || defined( UNDER_CE )
+#if defined( PTH_INIT_IN_PTH_H ) || defined( ST_INIT_IN_ST_H )
+#elif defined( WIN32 ) || defined( UNDER_CE )
     if ( !SetThreadPriority(GetCurrentThread(), i_priority) )
     {
         msg_Warn( p_this, "couldn't set a faster priority" );
@@ -694,20 +706,32 @@ int __vlc_thread_set_priority( vlc_object_t *p_this, char * psz_file,
     }
 
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
+
+#ifdef SYS_DARWIN
     if ( i_priority )
     {
-        int i_error;
+        int i_error, i_policy;
         struct sched_param param;
         memset( &param, 0, sizeof(struct sched_param) );
-        param.sched_priority = i_priority;
-        if ( (i_error = pthread_setschedparam( pthread_self(),
-                                               SCHED_RR, &param )) )
+        if ( i_priority < 0 )
         {
-            msg_Warn( p_this, "couldn't go to real-time priority (%s:%d): %s",
+            param.sched_priority = (-1) * i_priority;
+            i_policy = SCHED_OTHER;
+        }
+        else
+        {
+            param.sched_priority = i_priority;
+            i_policy = SCHED_RR;
+        }
+        if ( (i_error = pthread_setschedparam( pthread_self(),
+                                               i_policy, &param )) )
+        {
+            msg_Warn( p_this, "couldn't set thread priority (%s:%d): %s",     
                       psz_file, i_line, strerror(i_error) );
             i_priority = 0;
         }
     }
+#endif
 
 #endif
 
@@ -732,7 +756,7 @@ void __vlc_thread_join( vlc_object_t *p_this, char * psz_file, int i_line )
     int i_ret = 0;
 
 #if defined( PTH_INIT_IN_PTH_H )
-    i_ret = pth_join( p_this->thread_id, NULL );
+    i_ret = ( pth_join( p_this->thread_id, NULL ) == FALSE );
 
 #elif defined( ST_INIT_IN_ST_H )
     i_ret = st_thread_join( p_this->thread_id, NULL );
